@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (C) 2020 The Qt Company Ltd.
+# Copyright (C) 2024 The Qt Company Ltd.
 # SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 # import the print function which is used in python 3.x
@@ -10,20 +10,13 @@ import collections
 import os
 import shlex
 import shutil
+import sys
 
 import common
 
 def existing_path(path):
     return path if os.path.exists(path) else None
 
-def default_python3():
-    path_system = os.path.join('/usr', 'bin') if not common.is_windows_platform() else None
-    path = os.environ.get('PYTHON3_PATH') or path_system
-    postfix = '.exe' if common.is_windows_platform() else ''
-    return (path if not path
-            else (existing_path(os.path.join(path, 'python3' + postfix)) or
-                  existing_path(os.path.join(path, 'python' + postfix))))
-
 def get_arguments():
     parser = argparse.ArgumentParser(description='Build Qt Creator for packaging')
     parser.add_argument('--src', help='path to sources', required=True)
@@ -52,7 +45,7 @@ def get_arguments():
                         help='Path to python libraries for use by cdbextension (Windows)')
 
     parser.add_argument('--python3', help='File path to python3 executable for generating translations',
-                        default=default_python3())
+                        default=sys.executable)
 
     parser.add_argument('--no-qtcreator',
                         help='Skip Qt Creator build (only build separate tools)',
@@ -60,7 +53,8 @@ def get_arguments():
     parser.add_argument('--no-cdb',
                         help='Skip cdbextension and the python dependency packaging step (Windows)',
                         action='store_true', default=(not common.is_windows_platform()))
-    parser.add_argument('--no-qbs', help='Skip building Qbs as part of Qt Creator', action='store_true', default=False);
+    parser.add_argument('--no-qbs', help='Skip building Qbs as part of Qt Creator',
+                        action='store_true', default=False);
     parser.add_argument('--no-docs', help='Skip documentation generation',
                         action='store_true', default=False)
     parser.add_argument('--no-build-date', help='Does not show build date in about dialog, for reproducible builds',
@@ -75,6 +69,8 @@ def get_arguments():
                         action='store_true', default=False)
     parser.add_argument('--with-cpack', help='Create packages with cpack',
                         action='store_true', default=False)
+    parser.add_argument('--with-sdk-tool', help='Builds a internal sdk-tool (not standalone) which is used in Qt Design Studio builds',
+                        action='store_true', default=False)
     parser.add_argument('--add-path', help='Prepends a CMAKE_PREFIX_PATH to the build',
                         action='append', dest='prefix_paths', default=[])
     parser.add_argument('--add-module-path', help='Prepends a CMAKE_MODULE_PATH to the build',
@@ -146,6 +142,8 @@ def common_cmake_arguments(args):
     # Qt otherwise adds dependencies on libGLX and libOpenGL
     cmake_args += ['-DOpenGL_GL_PREFERENCE=LEGACY']
 
+    cmake_args += args.config_args
+
     return cmake_args
 
 def build_qtcreator(args, paths):
@@ -169,8 +167,10 @@ def build_qtcreator(args, paths):
                   '-DWITH_DOCS=' + cmake_option(not args.no_docs),
                   '-DBUILD_QBS=' + cmake_option(build_qbs),
                   '-DBUILD_DEVELOPER_DOCS=' + cmake_option(not args.no_docs),
-                  '-DBUILD_EXECUTABLE_SDKTOOL=OFF',
+                  '-DBUILD_LIBRARY_SDKTOOLLIB=' + cmake_option(args.with_sdk_tool),
+                  '-DBUILD_EXECUTABLE_SDKTOOL=' + cmake_option(args.with_sdk_tool),
                   '-DQTC_FORCE_XCB=ON',
+                  '-DQTC_USE_SYSTEM_LIBARCHIVE=OFF',
                   '-DWITH_TESTS=' + cmake_option(args.with_tests)]
     cmake_args += common_cmake_arguments(args)
 
@@ -194,8 +194,6 @@ def build_qtcreator(args, paths):
         if common.is_linux_platform():
             cmake_args += ['-DCPACK_INSTALL_PREFIX=/opt/qt-creator']
 
-    cmake_args += args.config_args
-
     common.check_print_call(cmake_args + [paths.src], paths.build)
     build_args = ['cmake', '--build', '.']
     if args.make_args:
@@ -245,7 +243,10 @@ def build_qtcreatorcdbext(args, paths):
         return
     if not os.path.exists(paths.qtcreatorcdbext_build):
         os.makedirs(paths.qtcreatorcdbext_build)
-    prefix_paths = [common.to_posix_path(os.path.abspath(fp)) for fp in args.prefix_paths]
+    prefix_paths = [os.path.abspath(fp) for fp in args.prefix_paths]
+    if paths.llvm:
+        prefix_paths += [paths.llvm]
+    prefix_paths = [common.to_posix_path(fp) for fp in prefix_paths]
     cmake_args = ['-DCMAKE_PREFIX_PATH=' + ';'.join(prefix_paths),
                   '-DCMAKE_INSTALL_PREFIX=' + common.to_posix_path(paths.qtcreatorcdbext_install)]
     cmake_args += common_cmake_arguments(args)
@@ -273,30 +274,31 @@ def package_qtcreator(args, paths):
         if not args.no_cdb:
             common.check_print_call(command + [paths.qtcreatorcdbext_install])
 
+    zip = common.sevenzip_command(args.zip_threads)
     if not args.no_zip:
         if not args.no_qtcreator:
-            common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                     os.path.join(paths.result, 'qtcreator' + args.zip_infix + '.7z'),
-                                     zipPatternForApp(paths)],
+            common.check_print_call(zip
+                                    + [os.path.join(paths.result, 'qtcreator' + args.zip_infix + '.7z'),
+                                       zipPatternForApp(paths)],
                                     paths.install)
-            common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                     os.path.join(paths.result, 'qtcreator' + args.zip_infix + '_dev.7z'),
-                                     '*'],
+            common.check_print_call(zip
+                                    + [os.path.join(paths.result, 'qtcreator' + args.zip_infix + '_dev.7z'),
+                                       '*'],
                                     paths.dev_install)
             if args.with_debug_info:
-                common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                         os.path.join(paths.result, 'qtcreator' + args.zip_infix + '-debug.7z'),
-                                         '*'],
+                common.check_print_call(zip
+                                        + [os.path.join(paths.result, 'qtcreator' + args.zip_infix + '-debug.7z'),
+                                           '*'],
                                         paths.debug_install)
         if common.is_windows_platform():
-            common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                     os.path.join(paths.result, 'wininterrupt' + args.zip_infix + '.7z'),
-                                     '*'],
+            common.check_print_call(zip
+                                    + [os.path.join(paths.result, 'wininterrupt' + args.zip_infix + '.7z'),
+                                       '*'],
                                     paths.wininterrupt_install)
             if not args.no_cdb:
-                common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                         os.path.join(paths.result, 'qtcreatorcdbext' + args.zip_infix + '.7z'),
-                                         '*'],
+                common.check_print_call(zip
+                                        + [os.path.join(paths.result, 'qtcreatorcdbext' + args.zip_infix + '.7z'),
+                                           '*'],
                                         paths.qtcreatorcdbext_install)
 
     if common.is_mac_platform() and not args.no_qtcreator:
@@ -310,12 +312,12 @@ def package_qtcreator(args, paths):
                 app = apps[0]
                 common.codesign(os.path.join(signed_install_path, app))
                 if not args.no_zip:
-                    common.check_print_call(['7z', 'a', '-mmt' + args.zip_threads,
-                                             os.path.join(paths.result, 'qtcreator' + args.zip_infix + '-signed.7z'),
-                                             app],
+                    common.check_print_call(zip
+                                            + [os.path.join(paths.result, 'qtcreator' + args.zip_infix + '-signed.7z'),
+                                               app],
                                             signed_install_path)
         if not args.no_dmg:
-            common.check_print_call(['python', '-u',
+            common.check_print_call([args.python3, '-u',
                                      os.path.join(paths.src, 'scripts', 'makedmg.py'),
                                      'qt-creator' + args.zip_infix + '.dmg',
                                      'Qt Creator',
