@@ -1,14 +1,16 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmljslink.h"
 
 #include "parser/qmljsast_p.h"
-#include "qmljsdocument.h"
+
 #include "qmljsbind.h"
-#include "qmljsutils.h"
-#include "qmljsmodelmanagerinterface.h"
 #include "qmljsconstants.h"
+#include "qmljsdocument.h"
+#include "qmljsmodelmanagerinterface.h"
+#include "qmljstr.h"
+#include "qmljsutils.h"
 
 #include <utils/algorithm.h>
 #include <utils/filepath.h>
@@ -83,6 +85,7 @@ public:
                               const Utils::FilePath &libraryPath);
     void loadImplicitDirectoryImports(Imports *imports, const Document::Ptr &doc);
     void loadImplicitDefaultImports(Imports *imports);
+    void loadImplicitBuiltinsImports(Imports *imports);
 
     void error(const Document::Ptr &doc, const SourceLocation &loc, const QString &message);
     void warning(const Document::Ptr &doc, const SourceLocation &loc, const QString &message);
@@ -91,6 +94,8 @@ public:
 private:
     friend class Link;
 
+    void loadImplicitImports(Imports *imports, const QString &packageName, const QString &moduleName);
+
     Snapshot m_snapshot;
     ValueOwner *m_valueOwner = nullptr;
     QList<Utils::FilePath> m_importPaths;
@@ -123,7 +128,8 @@ Link::Link(const Snapshot &snapshot, const ViewerContext &vContext, const Librar
 {
     d->m_valueOwner = new ValueOwner;
     d->m_snapshot = snapshot;
-    d->m_importPaths = vContext.paths;
+    const QList<Utils::FilePath> list(vContext.paths.begin(), vContext.paths.end());
+    d->m_importPaths = list;
     d->m_applicationDirectories = vContext.applicationDirectories;
     d->m_builtins = builtins;
     d->m_vContext = vContext;
@@ -190,12 +196,12 @@ Context::ImportsPerDocument LinkPrivate::linkImports()
         m_valueOwner->cppQmlTypes().load(QLatin1String("<builtins>"), m_builtins.metaObjects());
     } else {
         m_valueOwner->cppQmlTypes().load(QLatin1String("<defaults>"),
-                                         CppQmlTypesLoader::defaultQtObjects);
+                                         CppQmlTypesLoader::defaultQtObjects());
     }
 
     // load library objects shipped with Creator
     m_valueOwner->cppQmlTypes().load(QLatin1String("<defaultQt4>"),
-                                     CppQmlTypesLoader::defaultLibraryObjects);
+                                     CppQmlTypesLoader::defaultLibraryObjects());
 
     if (document) {
         // do it on document first, to make sure import errors are shown
@@ -203,9 +209,17 @@ Context::ImportsPerDocument LinkPrivate::linkImports()
 
         // Add custom imports for the opened document
         for (const auto &provider : CustomImportsProvider::allProviders()) {
-            const auto providerImports = provider->imports(m_valueOwner, document.data());
+            const auto providerImports = provider->imports(m_valueOwner,
+                                                           document.data(),
+                                                           &m_snapshot);
             for (const auto &import : providerImports)
                 importCache.insert(ImportCacheKey(import.info), import);
+            provider->loadBuiltins(&importsPerDocument,
+                                   imports,
+                                   document.data(),
+                                   m_valueOwner,
+                                   &m_snapshot);
+            m_importPaths = provider->prioritizeImportPaths(document.data(), m_importPaths);
         }
 
         populateImportedTypes(imports, document);
@@ -254,8 +268,11 @@ void LinkPrivate::populateImportedTypes(Imports *imports, const Document::Ptr &d
 {
     importableModuleApis.clear();
 
-    // implicit imports: the <default> package is always available
+    // implicit imports: the <default> package is always available (except when the QML package was loaded).
+    // In the latter case, still try to load the <default>'s module <defaults>.
     loadImplicitDefaultImports(imports);
+    // Load the <builtins> import, if the QML package was loaded.
+    loadImplicitBuiltinsImports(imports);
 
     // implicit imports:
     // qml files in the same directory are available without explicit imports
@@ -286,7 +303,7 @@ void LinkPrivate::populateImportedTypes(Imports *imports, const Document::Ptr &d
                 imports->setImportFailed();
                 if (info.ast()) {
                     error(doc, info.ast()->fileNameToken,
-                          Link::tr("File or directory not found."));
+                          Tr::tr("File or directory not found."));
                 }
                 break;
             default:
@@ -401,7 +418,7 @@ Import LinkPrivate::importNonFile(const Document::Ptr &doc, const ImportInfo &im
     const QString packageName = importInfo.name();
     const ComponentVersion version = importInfo.version();
 
-    QList<Utils::FilePath> libraryPaths = modulePaths(packageName,
+    const Utils::FilePaths libraryPaths = modulePaths(packageName,
                                                       version.toString(),
                                                       m_importPaths + m_applicationDirectories);
 
@@ -448,7 +465,7 @@ Import LinkPrivate::importNonFile(const Document::Ptr &doc, const ImportInfo &im
         error(doc,
               locationFromRange(importInfo.ast()->firstSourceLocation(),
                                 importInfo.ast()->lastSourceLocation()),
-              Link::tr(
+              Tr::tr(
                   "QML module not found (%1).\n\n"
                   "Import paths:\n"
                   "%2\n\n"
@@ -460,7 +477,7 @@ Import LinkPrivate::importNonFile(const Document::Ptr &doc, const ImportInfo &im
                   "For qmlRegister... calls, make sure that you define the Module URI as a string literal.\n")
                   .arg(importInfo.name(),
                        Utils::transform(m_importPaths, [](const Utils::FilePath &p) {
-                           return p.toString();
+                           return p.toUrlishString();
                        }).join("\n")));
     }
 
@@ -495,6 +512,16 @@ bool LinkPrivate::importLibrary(const Document::Ptr &doc,
     // commands in qmldir files, and is pending removal in Qt 6.
     for (const auto &toImport : libraryInfo.imports()) {
         QString importName = toImport.module;
+
+        // These implicit imports do not work reliable, since each type
+        // is only added to one import/module. If a type is added here,
+        // it is not added to the actual module, because of caching.
+        //
+        // In case of QtQuick3D.MaterialEditor this leads to a reproducible issue.
+        // As a workaround we simply skip all implicit imports for QtQuick3D.
+        if (importName == "QtQuick3D")
+            continue;
+
         ComponentVersion vNow = toImport.version;
         /* There was a period in which no version == auto
          * Required for QtQuick imports less than 2.15
@@ -511,29 +538,30 @@ bool LinkPrivate::importLibrary(const Document::Ptr &doc,
                                                                 m_importPaths);
         subImport.libraryPath = libraryPaths.value(0); // first is the best match
 
-        bool subImportFound = importLibrary(doc, subImport.libraryPath, &subImport, targetObject, importPath, true);
+        bool subImportFound = subImport.libraryPath != libraryPath
+                && importLibrary(doc, subImport.libraryPath, &subImport, targetObject, importPath, true);
 
         if (!subImportFound && errorLoc.isValid()) {
-            import->valid = false;
             if (!(optional || (toImport.flags & QmlDirParser::Import::Optional))) {
+                import->valid = false;
                 error(doc,
                       errorLoc,
-                      Link::tr("Implicit import '%1' of QML module '%2' not found.\n\n"
-                               "Import paths:\n"
-                               "%3\n\n"
-                               "For qmake projects, use the QML_IMPORT_PATH variable to add import "
-                               "paths.\n"
-                               "For Qbs projects, declare and set a qmlImportPaths property in "
-                               "your product "
-                               "to add import paths.\n"
-                               "For qmlproject projects, use the importPaths property to add "
-                               "import paths.\n"
-                               "For CMake projects, make sure QML_IMPORT_PATH variable is in "
-                               "CMakeCache.txt.\n")
-                          .arg(importName,
+                      Tr::tr("Implicit import '%1' of QML module '%2' not found.\n\n"
+                             "Import paths:\n"
+                             "%3\n\n"
+                             "For qmake projects, use the QML_IMPORT_PATH variable to add import "
+                             "paths.\n"
+                             "For Qbs projects, declare and set a qmlImportPaths property in "
+                             "your product "
+                             "to add import paths.\n"
+                             "For qmlproject projects, use the importPaths property to add "
+                             "import paths.\n"
+                             "For CMake projects, make sure QML_IMPORT_PATH variable is in "
+                             "CMakeCache.txt.\n")
+                      .arg(importName,
                                importInfo.name(),
                                Utils::transform(m_importPaths, [](const Utils::FilePath &p) {
-                                   return p.toString();
+                                   return p.toUrlishString();
                                }).join(QLatin1Char('\n'))));
             }
         } else if (!subImport.valid) {
@@ -562,8 +590,9 @@ bool LinkPrivate::importLibrary(const Document::Ptr &doc,
             if (!optional && errorLoc.isValid()) {
                 appendDiagnostic(doc, DiagnosticMessage(
                                      Severity::ReadingTypeInfoWarning, errorLoc,
-                                     Link::tr("QML module contains C++ plugins, "
-                                              "currently reading type information... %1").arg(import->info.name())));
+                                     Tr::tr("QML module contains C++ plugins, "
+                                            "currently reading type information... %1")
+                                     .arg(import->info.name())));
                 import->valid = false;
             }
         } else if (libraryInfo.pluginTypeInfoStatus() == LibraryInfo::DumpError
@@ -580,7 +609,7 @@ bool LinkPrivate::importLibrary(const Document::Ptr &doc,
             }
         } else {
             const QString packageName = importInfo.name();
-            m_valueOwner->cppQmlTypes().load(libraryPath.toString(),
+            m_valueOwner->cppQmlTypes().load(libraryPath.toUrlishString(),
                                              libraryInfo.metaObjects(),
                                              packageName);
             const auto objects = m_valueOwner->cppQmlTypes().createObjectsForImport(packageName,
@@ -646,14 +675,12 @@ void LinkPrivate::loadQmldirComponents(ObjectValue *import,
     QSet<QString> importedTypes;
     const auto components = libraryInfo.components();
     for (const QmlDirParser::Component &component : components) {
-        if (importedTypes.contains(component.typeName))
-            continue;
-
         ComponentVersion componentVersion(component.majorVersion, component.minorVersion);
         if (version < componentVersion)
             continue;
 
-        importedTypes.insert(component.typeName);
+        if (!Utils::insert(importedTypes, component.typeName))
+            continue;
         if (Document::Ptr importedDoc = m_snapshot.document(
                 libraryPath.pathAppended(component.fileName))) {
             if (ObjectValue *v = importedDoc->bind()->rootObjectValue())
@@ -683,20 +710,19 @@ void LinkPrivate::loadImplicitDirectoryImports(Imports *imports, const Document:
     }
 }
 
-void LinkPrivate::loadImplicitDefaultImports(Imports *imports)
+void LinkPrivate::loadImplicitImports(Imports *imports, const QString &packageName, const QString &moduleName)
 {
-    const QString defaultPackage = CppQmlTypes::defaultPackage;
-    if (m_valueOwner->cppQmlTypes().hasModule(defaultPackage)) {
+    if (m_valueOwner->cppQmlTypes().hasModule(packageName)) {
         const ComponentVersion maxVersion(ComponentVersion::MaxVersion,
                                           ComponentVersion::MaxVersion);
-        const ImportInfo info = ImportInfo::moduleImport(defaultPackage, maxVersion, QString());
+        const ImportInfo info = ImportInfo::moduleImport(packageName, maxVersion, QString());
         Import import = importCache.value(ImportCacheKey(info));
         if (!import.object) {
             import.valid = true;
             import.info = info;
-            import.object = new ObjectValue(m_valueOwner, QLatin1String("<defaults>"));
+            import.object = new ObjectValue(m_valueOwner, moduleName);
 
-            const auto objects = m_valueOwner->cppQmlTypes().createObjectsForImport(defaultPackage,
+            const auto objects = m_valueOwner->cppQmlTypes().createObjectsForImport(packageName,
                                                                                     maxVersion);
             for (const CppComponentValue *object : objects)
                 import.object->setMember(object->className(), object);
@@ -707,4 +733,14 @@ void LinkPrivate::loadImplicitDefaultImports(Imports *imports)
     }
 }
 
+void LinkPrivate::loadImplicitDefaultImports(Imports *imports)
+{
+    loadImplicitImports(imports, CppQmlTypes::defaultPackage, QLatin1String("<defaults>"));
+}
+
+void LinkPrivate::loadImplicitBuiltinsImports(Imports *imports)
+{
+    loadImplicitImports(imports, QLatin1String("QML"), QLatin1String("<builtins>"));
+}
+
 } // namespace QmlJS
