@@ -1,23 +1,25 @@
 // Copyright (C) 2022 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "deviceshell.h"
 
+#include "qtcprocess.h"
 #include "processinterface.h"
 #include "qtcassert.h"
-#include "qtcprocess.h"
+#include "utilstr.h"
 
 #include <QLoggingCategory>
-#include <QScopeGuard>
 
 Q_LOGGING_CATEGORY(deviceShellLog, "qtc.utils.deviceshell", QtWarningMsg)
 
+using namespace std::chrono_literals;
+
 namespace Utils {
 
-/*!
+/*
  * The multiplex script waits for input via stdin.
  *
- * To start a command, a message is send with
+ * To start a command, a message is sent with
  * the format "<cmd-id> "<base64-encoded-stdin-data>" <commandline>\n"
  * To stop the script, simply send "exit\n" via stdin
  *
@@ -62,7 +64,6 @@ DeviceShell::~DeviceShell()
  */
 RunResult DeviceShell::runInShell(const CommandLine &cmd, const QByteArray &stdInData)
 {
-    QTC_ASSERT(m_shellProcess, return {});
     Q_ASSERT(QThread::currentThread() != &m_thread);
 
     return run(cmd, stdInData);
@@ -74,51 +75,48 @@ QStringList DeviceShell::missingFeatures() const { return m_missingFeatures; }
 
 RunResult DeviceShell::run(const CommandLine &cmd, const QByteArray &stdInData)
 {
-    // If the script failed to install, use QtcProcess directly instead.
-    bool useProcess = m_shellScriptState == State::NoScript;
+    // If the script failed to install, use Process directly instead.
+    bool useProcess = m_shellScriptState == State::Failed;
 
     // Transferring large amounts of stdInData is slow via the shell script.
-    // Use QtcProcess directly if the size exceeds 100kb.
+    // Use Process directly if the size exceeds 100kb.
     useProcess |= stdInData.size() > (1024 * 100);
 
     if (useProcess) {
-        QtcProcess proc;
+        Process proc;
         const CommandLine fallbackCmd = createFallbackCommand(cmd);
         qCDebug(deviceShellLog) << "Running fallback:" << fallbackCmd;
         proc.setCommand(fallbackCmd);
         proc.setWriteData(stdInData);
-
-        proc.start();
-        proc.waitForFinished();
-
-        return RunResult{
-            proc.exitCode(),
-            proc.readAllStandardOutput(),
-            proc.readAllStandardError()
-        };
+        // Practically unlimited timeout since we are most probably running dd for copying
+        // data to a device here (=deployment), so a) we have no idea how long that might take
+        // for large files, and b) the user can cancel this manually.
+        proc.runBlocking(/*timout=*/1h);
+        // TODO This misses interesting data like proc.errorMessage() and/or proc.exitMessage()
+        return RunResult{proc.exitCode(), proc.rawStdOut(), proc.rawStdErr()};
     }
 
     const RunResult errorResult{-1, {}, {}};
     QTC_ASSERT(m_shellProcess, return errorResult);
+    QTC_ASSERT(m_shellProcess->isRunning(), return errorResult);
     QTC_ASSERT(m_shellScriptState == State::Succeeded, return errorResult);
 
     QMutexLocker lk(&m_commandMutex);
 
     QWaitCondition waiter;
     const int id = ++m_currentId;
-    const auto it = m_commandOutput.insert(id, CommandRun{{-1, {}, {}}, &waiter});
+    m_commandOutput.insert(id, CommandRun{{-1, {}, {}}, &waiter});
 
-    QMetaObject::invokeMethod(m_shellProcess.get(), [this, id, cmd, stdInData]() {
-        const QString command = QString("%1 \"%2\" %3\n")
-                                    .arg(id)
-                                    .arg(QString::fromLatin1(stdInData.toBase64()))
-                                    .arg(cmd.toUserOutput());
+    QMetaObject::invokeMethod(m_shellProcess.get(), [this, id, cmd, stdInData] {
+        const QString command = QString("%1 \"%2\" %3\n").arg(id)
+                                .arg(QString::fromLatin1(stdInData.toBase64()), cmd.toUserOutput());
         qCDebug(deviceShellLog) << "Running via shell:" << command;
         m_shellProcess->writeRaw(command.toUtf8());
     });
 
     waiter.wait(&m_commandMutex);
 
+    const auto it = m_commandOutput.constFind(id);
     const RunResult result = *it;
     m_commandOutput.erase(it);
 
@@ -140,7 +138,7 @@ void DeviceShell::close()
  * Override this function to setup the shell process.
  * The default implementation just sets the command line to "bash"
  */
-void DeviceShell::setupShellProcess(QtcProcess *shellProcess)
+void DeviceShell::setupShellProcess(Process *shellProcess)
 {
     shellProcess->setCommand(CommandLine{"bash"});
 }
@@ -157,122 +155,115 @@ CommandLine DeviceShell::createFallbackCommand(const CommandLine &cmd)
     return cmd;
 }
 
-/*!
- * \brief DeviceShell::startupFailed
- *
- * Override to display custom error messages
- */
-void DeviceShell::startupFailed(const CommandLine &cmdLine)
-{
-    qCWarning(deviceShellLog) << "Failed to start shell via:" << cmdLine.toUserOutput();
-}
-
 /*!
  * \brief DeviceShell::start
  * \return Returns true if starting the Shell process succeeded
  *
  * \note You have to call this function when deriving from DeviceShell. Current implementations call the function from their constructor.
  */
-bool DeviceShell::start()
+Result<> DeviceShell::start()
 {
-    m_shellProcess = std::make_unique<QtcProcess>();
-    connect(m_shellProcess.get(), &QtcProcess::done, m_shellProcess.get(),
+    m_shellProcess = std::make_unique<Process>();
+    connect(m_shellProcess.get(), &Process::done, m_shellProcess.get(),
             [this] { emit done(m_shellProcess->resultData()); });
     connect(&m_thread, &QThread::finished, m_shellProcess.get(), [this] { closeShellProcess(); }, Qt::DirectConnection);
 
     setupShellProcess(m_shellProcess.get());
 
-    CommandLine cmdLine = m_shellProcess->commandLine();
-
     m_shellProcess->setProcessMode(ProcessMode::Writer);
 
     // Moving the process into its own thread ...
     m_shellProcess->moveToThread(&m_thread);
 
-    bool result = false;
+    Result<> result;
     QMetaObject::invokeMethod(
         m_shellProcess.get(),
-        [this] {
-            qCDebug(deviceShellLog) << "Starting shell process:" << m_shellProcess->commandLine().toUserOutput();
+        [this]() -> Result<> {
+            qCDebug(deviceShellLog)
+                << "Starting shell process:" << m_shellProcess->commandLine().toUserOutput();
             m_shellProcess->start();
 
             if (!m_shellProcess->waitForStarted()) {
                 closeShellProcess();
-                return false;
+                return ResultError(Tr::tr("The process failed to start."));
             }
 
-            if (!installShellScript()) {
-                if (m_shellScriptState == State::FailedToStart)
-                    closeShellProcess();
-            } else {
+            auto installResult = installShellScript();
+            if (installResult) {
                 connect(m_shellProcess.get(),
-                        &QtcProcess::readyReadStandardOutput,
+                        &Process::readyReadStandardOutput,
                         m_shellProcess.get(),
                         [this] { onReadyRead(); });
                 connect(m_shellProcess.get(),
-                        &QtcProcess::readyReadStandardError,
+                        &Process::readyReadStandardError,
                         m_shellProcess.get(),
                         [this] {
-                            const QByteArray stdErr = m_shellProcess->readAllStandardError();
+                            const QByteArray stdErr = m_shellProcess->readAllRawStandardError();
                             qCWarning(deviceShellLog)
                                 << "Received unexpected output on stderr:" << stdErr;
                         });
+
+                connect(m_shellProcess.get(), &Process::done, m_shellProcess.get(), [this] {
+                    if (m_shellProcess->resultData().m_exitCode != EXIT_SUCCESS
+                        || m_shellProcess->resultData().m_exitStatus != QProcess::NormalExit) {
+                        qCWarning(deviceShellLog) << "Shell exited with error code:"
+                                                  << m_shellProcess->resultData().m_exitCode << "("
+                                                  << m_shellProcess->exitMessage() << ")";
+                    }
+                });
+
+                return {};
+            } else if (m_shellProcess->isRunning()) {
+                m_shellProcess->kill();
             }
-
-            connect(m_shellProcess.get(), &QtcProcess::done, m_shellProcess.get(), [this] {
-                if (m_shellProcess->resultData().m_exitCode != EXIT_SUCCESS
-                    || m_shellProcess->resultData().m_exitStatus != QProcess::NormalExit) {
-                    qCWarning(deviceShellLog) << "Shell exited with error code:"
-                                              << m_shellProcess->resultData().m_exitCode << "("
-                                              << m_shellProcess->exitMessage() << ")";
-                }
-            });
-
-            return true;
+            m_shellProcess.reset();
+            return ResultError(Tr::tr("Failed to install shell script: %1")
+                                       .arg(installResult.error()));
         },
         Qt::BlockingQueuedConnection,
         &result);
 
-    if (!result) {
-        startupFailed(cmdLine);
-    }
-
     return result;
 }
 
-bool DeviceShell::checkCommand(const QByteArray &command)
+Result<QByteArray> DeviceShell::checkCommand(const QByteArray &command)
 {
-    const QByteArray checkCmd = "(which " + command + " || echo '<missing>')\n";
+    const QByteArray checkCmd = "(type " + command + " || echo '<missing>')\n";
 
     m_shellProcess->writeRaw(checkCmd);
     if (!m_shellProcess->waitForReadyRead()) {
-        qCWarning(deviceShellLog) << "Timeout while trying to check for" << command;
-        return false;
+        if (!m_shellProcess->isRunning()) {
+            return ResultError(
+                m_shellProcess->exitMessage(Process::FailureMessageFormat::WithStdErr));
+        }
+        return ResultError(
+            Tr::tr("Timeout while trying to check for %1.").arg(QString::fromUtf8(command)));
     }
-    QByteArray out = m_shellProcess->readAllStandardOutput();
+    QByteArray out = m_shellProcess->readAllRawStandardOutput();
     if (out.contains("<missing>")) {
-        m_shellScriptState = State::NoScript;
-        qCWarning(deviceShellLog) << "Command" << command << "was not found";
+        m_shellScriptState = State::Failed;
         m_missingFeatures.append(QString::fromUtf8(command));
-        return false;
+        return ResultError(
+            Tr::tr("Command \"%1\" was not found.").arg(QString::fromUtf8(command)));
     }
 
-    return true;
+    return out;
 }
 
-bool DeviceShell::installShellScript()
+Result<> DeviceShell::installShellScript()
 {
     if (m_forceFailScriptInstallation) {
-        m_shellScriptState = State::NoScript;
-        return false;
+        m_shellScriptState = State::Failed;
+        return ResultError(Tr::tr("Script installation was forced to fail."));
     }
 
     static const QList<QByteArray> requiredCommands
         = {"base64", "cat", "echo", "kill", "mkfifo", "mktemp", "rm"};
 
     for (const QByteArray &command : requiredCommands) {
-        if (!checkCommand(command))
-            return false;
+        auto checkResult = checkCommand(command);
+        if (!checkResult)
+            return ResultError(checkResult.error());
     }
 
     const static QByteArray shellScriptBase64 = FilePath(":/utils/scripts/deviceshell.sh")
@@ -288,24 +279,26 @@ bool DeviceShell::installShellScript()
     m_shellProcess->writeRaw(scriptCmd);
 
     while (m_shellScriptState == State::Unknown) {
-        if (!m_shellProcess->waitForReadyRead(5000)) {
-            qCWarning(deviceShellLog) << "Timeout while waiting for shell script installation";
-            return false;
-        }
+        if (!m_shellProcess->waitForReadyRead(5s))
+            return ResultError(Tr::tr("Timeout while waiting for shell script installation."));
 
-        QByteArray out = m_shellProcess->readAllStandardError();
-        if (out.contains("SCRIPT_INSTALLED")) {
+        QByteArray out = m_shellProcess->readAllRawStandardError();
+        if (out.contains("SCRIPT_INSTALLED") && !out.contains("ERROR_INSTALL_SCRIPT")) {
             m_shellScriptState = State::Succeeded;
-            return true;
+            return ResultOk;
         }
         if (out.contains("ERROR_INSTALL_SCRIPT")) {
-            m_shellScriptState = State::NoScript;
-            qCWarning(deviceShellLog) << "Failed installing device shell script";
-            return false;
+            m_shellScriptState = State::Failed;
+            return ResultError(
+                Tr::tr("Failed to install shell script: %1").arg(QString::fromUtf8(out)));
+        }
+        if (!out.isEmpty()) {
+            qCDebug(deviceShellLog)
+                << "Unexpected output while installing device shell script:" << out;
         }
     }
 
-    return true;
+    return ResultOk;
 }
 
 void DeviceShell::closeShellProcess()
@@ -313,7 +306,7 @@ void DeviceShell::closeShellProcess()
     if (m_shellProcess) {
         if (m_shellProcess->isRunning()) {
             m_shellProcess->write("exit\nexit\n");
-            if (!m_shellProcess->waitForFinished(2000))
+            if (!m_shellProcess->waitForFinished(2s))
                 m_shellProcess->terminate();
         }
         m_shellProcess.reset();
@@ -399,7 +392,7 @@ QList<std::tuple<int, DeviceShell::ParseType, QByteArray>> parseShellOutput(cons
  */
 void DeviceShell::onReadyRead()
 {
-    m_commandBuffer += m_shellProcess->readAllStandardOutput();
+    m_commandBuffer += m_shellProcess->readAllRawStandardOutput();
     const qsizetype lastLineEndIndex = m_commandBuffer.lastIndexOf('\n') + 1;
 
     if (lastLineEndIndex == 0)
