@@ -1,60 +1,45 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include "autotestplugin.h"
-#include "autotesticons.h"
 #include "testresultspane.h"
-#include "testresultmodel.h"
+
+#include "autotesticons.h"
+#include "autotestplugin.h"
+#include "autotesttr.h"
+#include "itestframework.h"
+#include "testeditormark.h"
 #include "testresultdelegate.h"
+#include "testresultmodel.h"
+#include "testresultmodel.h"
 #include "testrunner.h"
 #include "testsettings.h"
 #include "testtreemodel.h"
-#include "testcodeparser.h"
-#include "testeditormark.h"
-#include "testoutputreader.h"
 
-#include <aggregation/aggregate.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <coreplugin/find/basetextfind.h>
 #include <coreplugin/find/itemviewfind.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/outputwindow.h>
+#include <coreplugin/session.h>
+
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/projectexplorer.h>
+
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditor.h>
 #include <texteditor/texteditorsettings.h>
+
+#include <utils/algorithm.h>
+#include <utils/fileutils.h>
+#include <utils/proxyaction.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 #include <utils/stylehelper.h>
 #include <utils/theme/theme.h>
 #include <utils/utilsicons.h>
 
-#include <QApplication>
-#include <QClipboard>
 #include <QDebug>
 #include <QFileDialog>
 #include <QHBoxLayout>
@@ -66,14 +51,16 @@
 #include <QVBoxLayout>
 
 using namespace Core;
+using namespace Utils;
 
-namespace Autotest {
-namespace Internal {
+namespace Autotest::Internal {
 
 ResultsTreeView::ResultsTreeView(QWidget *parent)
-    : Utils::TreeView(parent)
+    : TreeView(parent)
 {
+    setUniformRowHeights(false);
     setAttribute(Qt::WA_MacShowFocusRect, false);
+    setFrameStyle(NoFrame);
 }
 
 void ResultsTreeView::keyPressEvent(QKeyEvent *event)
@@ -89,6 +76,9 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     IOutputPane(parent),
     m_context(new IContext(this))
 {
+    setId("TestResults");
+    setDisplayName(Tr::tr("Test Results"));
+    setPriorityInStatusBar(-30);
     m_outputWidget = new QStackedWidget;
     QWidget *visualOutputWidget = new QWidget;
     m_outputWidget->addWidget(visualOutputWidget);
@@ -98,10 +88,8 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     visualOutputWidget->setLayout(outputLayout);
 
     QPalette pal;
-    pal.setColor(QPalette::Window,
-                 Utils::creatorTheme()->color(Utils::Theme::InfoBarBackground));
-    pal.setColor(QPalette::WindowText,
-                 Utils::creatorTheme()->color(Utils::Theme::InfoBarText));
+    pal.setColor(QPalette::Window, creatorColor(Theme::InfoBarBackground));
+    pal.setColor(QPalette::WindowText, creatorColor(Theme::InfoBarText));
     m_summaryWidget = new QFrame;
     m_summaryWidget->setPalette(pal);
     m_summaryWidget->setAutoFillBackground(true);
@@ -125,35 +113,42 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     m_model = new TestResultModel(this);
     m_filterModel = new TestResultFilterModel(m_model, this);
     m_filterModel->setDynamicSortFilter(true);
+    m_filterModel->setRecursiveFilteringEnabled(true);
     m_treeView->setModel(m_filterModel);
     TestResultDelegate *trd = new TestResultDelegate(this);
     m_treeView->setItemDelegate(trd);
 
     outputLayout->addWidget(ItemViewFind::createSearchableWrapper(m_treeView));
 
-    m_textOutput = new QPlainTextEdit;
-    m_textOutput->setPalette(pal);
-    m_textOutput->setFont(TextEditor::TextEditorSettings::fontSettings().font());
+    m_textOutput = new Core::OutputWindow(Core::Context("AutoTest.TextOutput"),
+                                          "AutoTest.TextOutput.Filter");
+
+    m_textOutput->setBaseFont(TextEditor::TextEditorSettings::fontSettings().font());
     m_textOutput->setWordWrapMode(QTextOption::WordWrap);
     m_textOutput->setReadOnly(true);
     m_outputWidget->addWidget(m_textOutput);
 
-    auto agg = new Aggregation::Aggregate;
-    agg->add(m_textOutput);
-    agg->add(new BaseTextFind(m_textOutput));
+    setupFilterUi("AutoTest.TextOutput.Filter", "Autotest::Internal::TestResultsPane");
+    setupContext("AutoTest.TextOutput", m_textOutput);
+    setFilteringEnabled(false);
+    setZoomButtonsEnabled(false);
+    connect(this, &IOutputPane::zoomInRequested, m_textOutput, &Core::OutputWindow::zoomIn);
+    connect(this, &IOutputPane::zoomOutRequested, m_textOutput, &Core::OutputWindow::zoomOut);
+    connect(this, &IOutputPane::resetZoomRequested, m_textOutput, &Core::OutputWindow::resetZoom);
+    connect(this, &IOutputPane::fontChanged, m_textOutput, &OutputWindow::setBaseFont);
 
     createToolButtons();
 
-    connect(m_treeView, &Utils::TreeView::activated, this, &TestResultsPane::onItemActivated);
+    connect(m_treeView, &TreeView::activated, this, &TestResultsPane::onItemActivated);
     connect(m_treeView->selectionModel(), &QItemSelectionModel::currentChanged,
             trd, &TestResultDelegate::currentChanged);
-    connect(m_treeView, &Utils::TreeView::customContextMenuRequested,
+    connect(m_treeView, &TreeView::customContextMenuRequested,
             this, &TestResultsPane::onCustomContextMenuRequested);
-    connect(m_treeView, &ResultsTreeView::copyShortcutTriggered, [this] () {
+    connect(m_treeView, &ResultsTreeView::copyShortcutTriggered, this, [this] {
         onCopyItemTriggered(getTestResult(m_treeView->currentIndex()));
     });
-    connect(m_model, &TestResultModel::requestExpansion, [this] (QModelIndex idx) {
-        m_treeView->expand(m_filterModel->mapFromSource(idx));
+    connect(m_model, &TestResultModel::requestExpansion, this, [this](const QModelIndex &idx) {
+        m_treeView->expandRecursively(m_filterModel->mapFromSource(idx));
     });
     connect(TestRunner::instance(), &TestRunner::testRunStarted,
             this, &TestResultsPane::onTestRunStarted);
@@ -163,16 +158,21 @@ TestResultsPane::TestResultsPane(QObject *parent) :
             this, &TestResultsPane::addTestResult);
     connect(TestRunner::instance(), &TestRunner::hadDisabledTests,
             m_model, &TestResultModel::raiseDisabledTests);
+    visualOutputWidget->installEventFilter(this);
+    connect(SessionManager::instance(), &SessionManager::sessionLoaded,
+            this, &TestResultsPane::onSessionLoaded);
+    connect(SessionManager::instance(), &SessionManager::aboutToSaveSession,
+            this, &TestResultsPane::onAboutToSaveSession);
 }
 
 void TestResultsPane::createToolButtons()
 {
     m_expandCollapse = new QToolButton(m_treeView);
     m_expandCollapse->setIcon(Utils::Icons::EXPAND_ALL_TOOLBAR.icon());
-    m_expandCollapse->setToolTip(tr("Expand All"));
+    m_expandCollapse->setToolTip(Tr::tr("Expand All"));
     m_expandCollapse->setCheckable(true);
     m_expandCollapse->setChecked(false);
-    connect(m_expandCollapse, &QToolButton::clicked, [this] (bool checked) {
+    connect(m_expandCollapse, &QToolButton::clicked, this, [this](bool checked) {
         if (checked)
             m_treeView->expandAll();
         else
@@ -180,24 +180,34 @@ void TestResultsPane::createToolButtons()
     });
 
     m_runAll = new QToolButton(m_treeView);
-    m_runAll->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action());
+    m_runAll->setDefaultAction(ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action(),
+                    Utils::Icons::RUN_SMALL_TOOLBAR.icon()));
 
     m_runSelected = new QToolButton(m_treeView);
-    m_runSelected->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action());
+    m_runSelected->setDefaultAction(ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action(),
+                    Utils::Icons::RUN_SELECTED_TOOLBAR.icon()));
 
+    m_runFailed = new QToolButton(m_treeView);
+    m_runFailed->setDefaultAction(ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FAILED_ID)->action(),
+                    Icons::RUN_FAILED_TOOLBAR.icon()));
     m_runFile = new QToolButton(m_treeView);
-    m_runFile->setDefaultAction(ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action());
+    m_runFile->setDefaultAction(ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action(),
+                    Utils::Icons::RUN_FILE_TOOLBAR.icon()));
 
     m_stopTestRun = new QToolButton(m_treeView);
     m_stopTestRun->setIcon(Utils::Icons::STOP_SMALL_TOOLBAR.icon());
-    m_stopTestRun->setToolTip(tr("Stop Test Run"));
+    m_stopTestRun->setToolTip(Tr::tr("Stop Test Run"));
     m_stopTestRun->setEnabled(false);
     connect(m_stopTestRun, &QToolButton::clicked, TestRunner::instance(), &TestRunner::requestStopTestRun);
 
     m_filterButton = new QToolButton(m_treeView);
     m_filterButton->setIcon(Utils::Icons::FILTER.icon());
-    m_filterButton->setToolTip(tr("Filter Test Results"));
-    m_filterButton->setProperty("noArrow", true);
+    m_filterButton->setToolTip(Tr::tr("Filter Test Results"));
+    m_filterButton->setProperty(StyleHelper::C_NO_ARROW, true);
     m_filterButton->setPopupMode(QToolButton::InstantPopup);
     m_filterMenu = new QMenu(m_filterButton);
     initializeFilterMenu();
@@ -205,9 +215,26 @@ void TestResultsPane::createToolButtons()
     m_filterButton->setMenu(m_filterMenu);
     m_outputToggleButton = new QToolButton(m_treeView);
     m_outputToggleButton->setIcon(Icons::TEXT_DISPLAY.icon());
-    m_outputToggleButton->setToolTip(tr("Switch Between Visual and Text Display"));
+    m_outputToggleButton->setToolTip(Tr::tr("Switch Between Visual and Text Display"));
     m_outputToggleButton->setEnabled(true);
     connect(m_outputToggleButton, &QToolButton::clicked, this, &TestResultsPane::toggleOutputStyle);
+    m_showDurationButton = new QToolButton(m_treeView);
+    auto icon = Utils::Icon({{":/utils/images/stopwatch.png", Utils::Theme::IconsBaseColor}});
+    m_showDurationButton->setIcon(icon.icon());
+    m_showDurationButton->setToolTip(Tr::tr("Show Durations"));
+    m_showDurationButton->setCheckable(true);
+    m_showDurationButton->setChecked(true);
+    connect(m_showDurationButton, &QToolButton::toggled, this, [this](bool checked) {
+        if (auto trd = qobject_cast<TestResultDelegate *>(m_treeView->itemDelegate())) {
+            trd->setShowDuration(checked);
+            if (m_model->rowCount()) {
+                m_model->rootItem()->forAllChildren([this](TestResultItem *it) {
+                    const QModelIndex idx = m_model->indexForItem(it);
+                    emit m_model->dataChanged(idx, idx, {Qt::DisplayRole});
+                });
+            }
+        }
+    });
 }
 
 static TestResultsPane *s_instance = nullptr;
@@ -227,7 +254,7 @@ TestResultsPane::~TestResultsPane()
     s_instance = nullptr;
 }
 
-void TestResultsPane::addTestResult(const TestResultPtr &result)
+void TestResultsPane::addTestResult(const TestResult &result)
 {
     const QScrollBar *scrollBar = m_treeView->verticalScrollBar();
     m_atEnd = scrollBar ? scrollBar->value() == scrollBar->maximum() : true;
@@ -240,33 +267,6 @@ void TestResultsPane::addTestResult(const TestResultPtr &result)
     navigateStateChanged();
 }
 
-static void checkAndFineTuneColors(QTextCharFormat *format)
-{
-    QTC_ASSERT(format, return);
-    const QColor bgColor = format->background().color();
-    QColor fgColor = format->foreground().color();
-
-    if (Utils::StyleHelper::isReadableOn(bgColor, fgColor))
-        return;
-
-    int h, s, v;
-    fgColor.getHsv(&h, &s, &v);
-    // adjust the color value to ensure better readability
-    if (Utils::StyleHelper::luminance(bgColor) < .5)
-        v = v + 64;
-    else
-        v = v - 64;
-
-    fgColor.setHsv(h, s, v);
-    if (!Utils::StyleHelper::isReadableOn(bgColor, fgColor)) {
-        s = (s + 128) % 255;    // adjust the saturation to ensure better readability
-        fgColor.setHsv(h, s, v);
-        if (!Utils::StyleHelper::isReadableOn(bgColor, fgColor))
-            return;
-    }
-
-    format->setForeground(fgColor);
-}
 
 void TestResultsPane::addOutputLine(const QByteArray &outputLine, OutputChannel channel)
 {
@@ -276,20 +276,9 @@ void TestResultsPane::addOutputLine(const QByteArray &outputLine, OutputChannel
         return;
     }
 
-    const Utils::FormattedText formattedText
-            = Utils::FormattedText{QString::fromUtf8(outputLine), m_defaultFormat};
-    QList<Utils::FormattedText> formatted = channel == OutputChannel::StdOut
-            ? m_stdOutHandler.parseText(formattedText)
-            : m_stdErrHandler.parseText(formattedText);
-
-    QTextCursor cursor = m_textOutput->textCursor();
-    cursor.beginEditBlock();
-    for (auto formattedText : formatted) {
-        checkAndFineTuneColors(&formattedText.format);
-        cursor.insertText(formattedText.text, formattedText.format);
-    }
-    cursor.insertText("\n");
-    cursor.endEditBlock();
+    m_textOutput->appendMessage(QString::fromUtf8(outputLine) + '\n',
+                                channel == OutputChannel::StdOut ? OutputFormat::StdOutFormat
+                                                                 : OutputFormat::StdErrFormat);
 }
 
 QWidget *TestResultsPane::outputWidget(QWidget *parent)
@@ -304,18 +293,12 @@ QWidget *TestResultsPane::outputWidget(QWidget *parent)
 
 QList<QWidget *> TestResultsPane::toolBarWidgets() const
 {
-    return {m_expandCollapse, m_runAll, m_runSelected, m_runFile, m_stopTestRun,
-            m_outputToggleButton, m_filterButton};
-}
-
-QString TestResultsPane::displayName() const
-{
-    return tr("Test Results");
-}
-
-int TestResultsPane::priorityInStatusBar() const
-{
-    return -666;
+    QList<QWidget *> result = {m_expandCollapse, m_runAll, m_runSelected, m_runFailed,
+                               m_runFile, m_stopTestRun, m_showDurationButton,
+                               m_outputToggleButton, m_filterButton};
+    for (QWidget *widget : IOutputPane::toolBarWidgets())
+        result.append(widget);
+    return result;
 }
 
 void TestResultsPane::clearContents()
@@ -326,25 +309,13 @@ void TestResultsPane::clearContents()
     setIconBadgeNumber(0);
     navigateStateChanged();
     m_summaryWidget->setVisible(false);
-    m_autoScroll = AutotestPlugin::settings()->autoScroll;
+    m_autoScroll = testSettings().autoScroll();
     connect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
             this, &TestResultsPane::onScrollBarRangeChanged, Qt::UniqueConnection);
     m_textOutput->clear();
-    m_defaultFormat.setBackground(Utils::creatorTheme()->palette().color(
-                                      m_textOutput->backgroundRole()));
-    m_defaultFormat.setForeground(Utils::creatorTheme()->palette().color(
-                                      m_textOutput->foregroundRole()));
-
-    // in case they had been forgotten to reset
-    m_stdErrHandler.endFormatScope();
-    m_stdOutHandler.endFormatScope();
     clearMarks();
 }
 
-void TestResultsPane::visibilityChanged(bool /*visible*/)
-{
-}
-
 void TestResultsPane::setFocus()
 {
 }
@@ -404,7 +375,7 @@ void TestResultsPane::goToNext()
 
     // if we have no current or could not find a next one, use the first item of the whole tree
     if (!nextCurrentIndex.isValid()) {
-        Utils::TreeItem *rootItem = m_model->itemForIndex(QModelIndex());
+        TreeItem *rootItem = m_model->itemForIndex(QModelIndex());
         // if the tree does not contain any item - don't do anything
         if (!rootItem || !rootItem->childCount())
             return;
@@ -454,95 +425,87 @@ void TestResultsPane::goToPrev()
     onItemActivated(nextCurrentIndex);
 }
 
+void TestResultsPane::updateFilter()
+{
+    m_textOutput->updateFilterProperties(filterText(), filterCaseSensitivity(), filterUsesRegexp(),
+                                         filterIsInverted(), beforeContext(), afterContext());
+}
+
 void TestResultsPane::onItemActivated(const QModelIndex &index)
 {
     if (!index.isValid())
         return;
 
-    const TestResult *testResult = m_filterModel->testResult(index);
-    if (testResult && !testResult->fileName().isEmpty())
-        EditorManager::openEditorAt(testResult->fileName(), testResult->line(), 0);
-}
-
-void TestResultsPane::onRunAllTriggered()
-{
-    TestRunner *runner = TestRunner::instance();
-    runner->setSelectedTests(TestTreeModel::instance()->getAllTestCases());
-    runner->prepareToRunTests(TestRunMode::Run);
-}
-
-void TestResultsPane::onRunSelectedTriggered()
-{
-    TestRunner *runner = TestRunner::instance();
-    runner->setSelectedTests(TestTreeModel::instance()->getSelectedTests());
-    runner->prepareToRunTests(TestRunMode::Run);
+    const TestResult testResult = m_filterModel->testResult(index);
+    if (testResult.isValid() && !testResult.fileName().isEmpty())
+        EditorManager::openEditorAt(Link{testResult.fileName(), testResult.line(), 0});
 }
 
 void TestResultsPane::initializeFilterMenu()
 {
-    const bool omitIntern = AutotestPlugin::settings()->omitInternalMssg;
-    // FilterModel has all messages enabled by default
-    if (omitIntern)
-        m_filterModel->toggleTestResultType(ResultType::MessageInternal);
-
     QMap<ResultType, QString> textAndType;
-    textAndType.insert(ResultType::Pass, tr("Pass"));
-    textAndType.insert(ResultType::Fail, tr("Fail"));
-    textAndType.insert(ResultType::ExpectedFail, tr("Expected Fail"));
-    textAndType.insert(ResultType::UnexpectedPass, tr("Unexpected Pass"));
-    textAndType.insert(ResultType::Skip, tr("Skip"));
-    textAndType.insert(ResultType::Benchmark, tr("Benchmarks"));
-    textAndType.insert(ResultType::MessageDebug, tr("Debug Messages"));
-    textAndType.insert(ResultType::MessageWarn, tr("Warning Messages"));
-    textAndType.insert(ResultType::MessageInternal, tr("Internal Messages"));
-    for (ResultType result : textAndType.keys()) {
+    textAndType.insert(ResultType::Pass, Tr::tr("Pass"));
+    textAndType.insert(ResultType::Fail, Tr::tr("Fail"));
+    textAndType.insert(ResultType::ExpectedFail, Tr::tr("Expected Fail"));
+    textAndType.insert(ResultType::UnexpectedPass, Tr::tr("Unexpected Pass"));
+    textAndType.insert(ResultType::Skip, Tr::tr("Skip"));
+    textAndType.insert(ResultType::Benchmark, Tr::tr("Benchmarks"));
+    textAndType.insert(ResultType::MessageDebug, Tr::tr("Debug Messages"));
+    textAndType.insert(ResultType::MessageWarn, Tr::tr("Warning Messages"));
+    textAndType.insert(ResultType::MessageInternal, Tr::tr("Internal Messages"));
+    const QSet<ResultType> enabled = m_filterModel->enabledFilters();
+    for (auto it = textAndType.cbegin(); it != textAndType.cend(); ++it) {
+        const ResultType &result = it.key();
         QAction *action = new QAction(m_filterMenu);
-        action->setText(textAndType.value(result));
+        action->setText(it.value());
         action->setCheckable(true);
-        action->setChecked(result != ResultType::MessageInternal || !omitIntern);
+        action->setChecked(enabled.contains(result));
         action->setData(int(result));
         m_filterMenu->addAction(action);
     }
     m_filterMenu->addSeparator();
-    QAction *action = new QAction(tr("Check All Filters"), m_filterMenu);
+    QAction *action = new QAction(Tr::tr("Check All Filters"), m_filterMenu);
     m_filterMenu->addAction(action);
-    connect(action, &QAction::triggered, this, [this]() { TestResultsPane::checkAllFilter(true); });
-    action = new QAction(tr("Uncheck All Filters"), m_filterMenu);
+    connect(action, &QAction::triggered, this, [this] { TestResultsPane::checkAllFilter(true); });
+    action = new QAction(Tr::tr("Uncheck All Filters"), m_filterMenu);
     m_filterMenu->addAction(action);
-    connect(action, &QAction::triggered, this, [this]() { TestResultsPane::checkAllFilter(false); });
+    connect(action, &QAction::triggered, this, [this] { TestResultsPane::checkAllFilter(false); });
 }
 
 void TestResultsPane::updateSummaryLabel()
 {
     QString labelText = QString("<p>");
-    labelText.append(tr("Test summary"));
+    labelText.append(Tr::tr("Test summary"));
     labelText.append(":&nbsp;&nbsp; ");
     int count = m_model->resultTypeCount(ResultType::Pass);
-    labelText += QString::number(count) + ' ' + tr("passes");
+    labelText += QString::number(count) + ' ' + Tr::tr("passes");
     count = m_model->resultTypeCount(ResultType::Fail);
-    labelText += ", " + QString::number(count) + ' ' + tr("fails");
+    labelText += ", " + QString::number(count) + ' ' + Tr::tr("fails");
     count = m_model->resultTypeCount(ResultType::UnexpectedPass);
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("unexpected passes");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("unexpected passes");
     count = m_model->resultTypeCount(ResultType::ExpectedFail);
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("expected fails");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("expected fails");
     count = m_model->resultTypeCount(ResultType::MessageFatal);
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("fatals");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("fatals");
     count = m_model->resultTypeCount(ResultType::BlacklistedFail)
             + m_model->resultTypeCount(ResultType::BlacklistedXFail)
             + m_model->resultTypeCount(ResultType::BlacklistedPass)
             + m_model->resultTypeCount(ResultType::BlacklistedXPass);
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("blacklisted");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("blacklisted");
     count = m_model->resultTypeCount(ResultType::Skip);
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("skipped");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("skipped");
     count = m_model->disabledTests();
     if (count)
-        labelText += ", " + QString::number(count) + ' ' + tr("disabled");
-    labelText.append(".</p>");
+        labelText += ", " + QString::number(count) + ' ' + Tr::tr("disabled");
+    if (auto millisec = m_model->reportedDuration())
+        labelText += ".&nbsp;&nbsp;&nbsp;(" + QString::number(*millisec) + " ms)</p>";
+    else
+        labelText.append(".</p>");
     m_summaryLabel->setText(labelText);
 }
 
@@ -561,11 +524,19 @@ void TestResultsPane::filterMenuTriggered(QAction *action)
     navigateStateChanged();
 }
 
+bool TestResultsPane::eventFilter(QObject *object, QEvent *event)
+{
+    QTC_ASSERT(m_outputWidget, return false);
+    if (event->type() == QEvent::Resize && object->parent() == m_outputWidget)
+        static_cast<TestResultDelegate *>(m_treeView->itemDelegate())->clearCache();
+    return false;
+}
+
 void TestResultsPane::onTestRunStarted()
 {
     m_testRunning = true;
     m_stopTestRun->setEnabled(true);
-    AutotestPlugin::updateMenuItemsEnabledState();
+    updateMenuItemsEnabledState();
     m_summaryWidget->setVisible(false);
 }
 
@@ -581,14 +552,13 @@ void TestResultsPane::onTestRunFinished()
     m_testRunning = false;
     m_stopTestRun->setEnabled(false);
 
-    AutotestPlugin::updateMenuItemsEnabledState();
+    updateMenuItemsEnabledState();
     updateSummaryLabel();
     m_summaryWidget->setVisible(true);
     m_model->removeCurrentTestMessage();
     disconnect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
                this, &TestResultsPane::onScrollBarRangeChanged);
-    if (AutotestPlugin::settings()->popupOnFinish
-            && (!AutotestPlugin::settings()->popupOnFail || hasFailedTests(m_model))) {
+    if (testSettings().popupOnFinish() && (!testSettings().popupOnFail() || hasFailedTests(m_model))) {
         popup(IOutputPane::NoModeSwitch);
     }
     createMarks();
@@ -604,88 +574,105 @@ void TestResultsPane::onCustomContextMenuRequested(const QPoint &pos)
 {
     const bool resultsAvailable = m_filterModel->hasResults();
     const bool enabled = !m_testRunning && resultsAvailable;
-    const TestResult *clicked = getTestResult(m_treeView->indexAt(pos));
+    const TestResult clicked = getTestResult(m_treeView->indexAt(pos));
     QMenu menu;
 
-    QAction *action = new QAction(tr("Copy"), &menu);
+    QAction *action = new QAction(Tr::tr("Copy"), &menu);
     action->setShortcut(QKeySequence(QKeySequence::Copy));
-    action->setEnabled(resultsAvailable && clicked);
-    connect(action, &QAction::triggered, [this, clicked] () {
+    action->setEnabled(resultsAvailable && clicked.isValid());
+    connect(action, &QAction::triggered, this, [this, &clicked] {
        onCopyItemTriggered(clicked);
     });
     menu.addAction(action);
 
-    action = new QAction(tr("Copy All"), &menu);
+    action = new QAction(Tr::tr("Copy All"), &menu);
     action->setEnabled(enabled);
     connect(action, &QAction::triggered, this, &TestResultsPane::onCopyWholeTriggered);
     menu.addAction(action);
 
-    action = new QAction(tr("Save Output to File..."), &menu);
+    action = new QAction(Tr::tr("Save Output to File..."), &menu);
     action->setEnabled(enabled);
     connect(action, &QAction::triggered, this, &TestResultsPane::onSaveWholeTriggered);
     menu.addAction(action);
 
-    const auto correlatingItem = (enabled && clicked) ? clicked->findTestTreeItem() : nullptr;
-    action = new QAction(tr("Run This Test"), &menu);
+    const auto correlatingItem = (enabled && clicked.isValid()) ? clicked.findTestTreeItem() : nullptr;
+    action = new QAction(Tr::tr("Run This Test"), &menu);
     action->setEnabled(correlatingItem && correlatingItem->canProvideTestConfiguration());
-    connect(action, &QAction::triggered, this, [this, clicked] {
+    connect(action, &QAction::triggered, this, [this, &clicked] {
         onRunThisTestTriggered(TestRunMode::Run, clicked);
     });
     menu.addAction(action);
 
-    action = new QAction(tr("Debug This Test"), &menu);
-    action->setEnabled(correlatingItem && correlatingItem->canProvideDebugConfiguration());
-    connect(action, &QAction::triggered, this, [this, clicked] {
+    action = new QAction(Tr::tr("Run This Test Without Deployment"), &menu);
+    action->setEnabled(correlatingItem && correlatingItem->canProvideTestConfiguration());
+    connect(action, &QAction::triggered, this, [this, &clicked] {
+        onRunThisTestTriggered(TestRunMode::RunWithoutDeploy, clicked);
+    });
+    menu.addAction(action);
+
+    action = new QAction(Tr::tr("Debug This Test"), &menu);
+    bool debugEnabled = false;
+    if (correlatingItem) {
+        if (correlatingItem->testBase()->type() == ITestBase::Framework) {
+            auto testTreeItem = static_cast<const TestTreeItem *>(correlatingItem);
+            debugEnabled = testTreeItem && testTreeItem->canProvideDebugConfiguration();
+        }
+    }
+    action->setEnabled(debugEnabled);
+    connect(action, &QAction::triggered, this, [this, &clicked] {
         onRunThisTestTriggered(TestRunMode::Debug, clicked);
     });
     menu.addAction(action);
 
+    action = new QAction(Tr::tr("Debug This Test Without Deployment"), &menu);
+    action->setEnabled(debugEnabled);
+    connect(action, &QAction::triggered, this, [this, &clicked] {
+        onRunThisTestTriggered(TestRunMode::DebugWithoutDeploy, clicked);
+    });
+    menu.addAction(action);
+
     menu.exec(m_treeView->mapToGlobal(pos));
 }
 
-const TestResult *TestResultsPane::getTestResult(const QModelIndex &idx)
+TestResult TestResultsPane::getTestResult(const QModelIndex &idx)
 {
     if (!idx.isValid())
-        return nullptr;
-
-    const TestResult *result = m_filterModel->testResult(idx);
-    QTC_CHECK(result);
-
+        return {};
+    const TestResult result = m_filterModel->testResult(idx);
+    QTC_CHECK(result.isValid());
     return result;
 }
 
-void TestResultsPane::onCopyItemTriggered(const TestResult *result)
+void TestResultsPane::onCopyItemTriggered(const TestResult &result)
 {
-    QTC_ASSERT(result, return);
-    QApplication::clipboard()->setText(result->outputString(true));
+    QTC_ASSERT(result.isValid(), return);
+    setClipboardAndSelection(result.outputString(true));
 }
 
 void TestResultsPane::onCopyWholeTriggered()
 {
-    QApplication::clipboard()->setText(getWholeOutput());
+    setClipboardAndSelection(getWholeOutput());
 }
 
 void TestResultsPane::onSaveWholeTriggered()
 {
-    const QString fileName = QFileDialog::getSaveFileName(ICore::dialogParent(),
-                                                          tr("Save Output To"));
-    if (fileName.isEmpty())
+    const FilePath filePath = FileUtils::getSaveFilePath(Tr::tr("Save Output To"));
+    if (filePath.isEmpty())
         return;
 
-    Utils::FileSaver saver(fileName, QIODevice::Text);
+    FileSaver saver(filePath, QIODevice::Text);
     if (!saver.write(getWholeOutput().toUtf8()) || !saver.finalize()) {
-        QMessageBox::critical(ICore::dialogParent(), tr("Error"),
-                              tr("Failed to write \"%1\".\n\n%2").arg(fileName)
+        QMessageBox::critical(ICore::dialogParent(), Tr::tr("Error"),
+                              Tr::tr("Failed to write \"%1\".\n\n%2").arg(filePath.toUserOutput())
                               .arg(saver.errorString()));
     }
 }
 
-void TestResultsPane::onRunThisTestTriggered(TestRunMode runMode, const TestResult *result)
+void TestResultsPane::onRunThisTestTriggered(TestRunMode runMode, const TestResult &result)
 {
-    QTC_ASSERT(result, return);
-
-    const TestTreeItem *item = result->findTestTreeItem();
+    QTC_ASSERT(result.isValid(), return);
 
+    const ITestTreeItem *item = result.findTestTreeItem();
     if (item)
         TestRunner::instance()->runTest(runMode, item);
 }
@@ -696,6 +683,8 @@ void TestResultsPane::toggleOutputStyle()
     m_outputWidget->setCurrentIndex(displayText ? 1 : 0);
     m_outputToggleButton->setIcon(displayText ? Icons::VISUAL_DISPLAY.icon()
                                               : Icons::TEXT_DISPLAY.icon());
+    setFilteringEnabled(displayText);
+    setZoomButtonsEnabled(displayText);
 }
 
 // helper for onCopyWholeTriggered() and onSaveWholeTriggered()
@@ -704,11 +693,11 @@ QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
     QString output;
     for (int row = 0, count = m_model->rowCount(parent); row < count; ++row) {
         QModelIndex current = m_model->index(row, 0, parent);
-        const TestResult *result = m_model->testResult(current);
-        QTC_ASSERT(result, continue);
+        const TestResult result = m_model->testResult(current);
+        QTC_ASSERT(result.isValid(), continue);
         if (auto item = m_model->itemForIndex(current))
             output.append(item->resultString()).append('\t');
-        output.append(result->outputString(true)).append('\n');
+        output.append(result.outputString(true)).append('\n');
         output.append(getWholeOutput(current));
     }
     return output;
@@ -716,26 +705,25 @@ QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
 
 void TestResultsPane::createMarks(const QModelIndex &parent)
 {
-    const TestResult *parentResult = m_model->testResult(parent);
-    ResultType parentType = parentResult ? parentResult->result() : ResultType::Invalid;
-    const QVector<ResultType> interested{ResultType::Fail, ResultType::UnexpectedPass};
+    const TestResult parentResult = m_model->testResult(parent);
+    const ResultType parentType = parentResult.isValid() ? parentResult.result() : ResultType::Invalid;
+    const QList<ResultType> interested{ResultType::Fail, ResultType::UnexpectedPass};
     for (int row = 0, count = m_model->rowCount(parent); row < count; ++row) {
         const QModelIndex index = m_model->index(row, 0, parent);
-        const TestResult *result = m_model->testResult(index);
-        QTC_ASSERT(result, continue);
+        const TestResult result = m_model->testResult(index);
+        QTC_ASSERT(result.isValid(), continue);
 
         if (m_model->hasChildren(index))
             createMarks(index);
 
-        bool isLocationItem = result->result() == ResultType::MessageLocation;
-        if (interested.contains(result->result())
+        bool isLocationItem = result.result() == ResultType::MessageLocation;
+        if (interested.contains(result.result())
                 || (isLocationItem && interested.contains(parentType))) {
-            const Utils::FilePath fileName = Utils::FilePath::fromString(result->fileName());
-            TestEditorMark *mark = new TestEditorMark(index, fileName, result->line());
+            TestEditorMark *mark = new TestEditorMark(index, result.fileName(), result.line());
             mark->setIcon(index.data(Qt::DecorationRole).value<QIcon>());
-            mark->setColor(Utils::Theme::OutputPanes_TestFailTextColor);
+            mark->setColor(Theme::OutputPanes_TestFailTextColor);
             mark->setPriority(TextEditor::TextMark::NormalPriority);
-            mark->setToolTip(result->description());
+            mark->setToolTip(result.description());
             m_marks << mark;
         }
     }
@@ -747,6 +735,34 @@ void TestResultsPane::clearMarks()
     m_marks.clear();
 }
 
+static constexpr char SV_SHOW_DURATIONS[] = "AutoTest.ShowDurations";
+static constexpr char SV_MESSAGE_FILTER[] = "AutoTest.MessageFilter";
+
+void TestResultsPane::onSessionLoaded()
+{
+    const bool showDurations = SessionManager::sessionValue(SV_SHOW_DURATIONS, true).toBool();
+    m_showDurationButton->setChecked(showDurations);
+    const QVariantList enabledFilters = SessionManager::sessionValue(SV_MESSAGE_FILTER).toList();
+
+    if (enabledFilters.isEmpty()) {
+        m_filterModel->enableAllResultTypes(true);
+        if (testSettings().omitInternalMsg())
+            m_filterModel->toggleTestResultType(ResultType::MessageInternal);
+    } else {
+        m_filterModel->setEnabledFiltersFromSetting(enabledFilters);
+    }
+
+    m_filterMenu->clear();
+    initializeFilterMenu();
+}
+
+
+void TestResultsPane::onAboutToSaveSession()
+{
+    SessionManager::setSessionValue(SV_SHOW_DURATIONS, m_showDurationButton->isChecked());
+    SessionManager::setSessionValue(SV_MESSAGE_FILTER, m_filterModel->enabledFiltersAsSetting());
+}
+
 void TestResultsPane::showTestResult(const QModelIndex &index)
 {
     QModelIndex mapped = m_filterModel->mapFromSource(index);
@@ -756,5 +772,4 @@ void TestResultsPane::showTestResult(const QModelIndex &index)
     }
 }
 
-} // namespace Internal
-} // namespace Autotest
+} // namespace Autotest::Internal
