@@ -21,16 +21,13 @@
 #include <coreplugin/icore.h>
 #include <cplusplus/AST.h>
 #include <cplusplus/ASTPath.h>
-#include <cplusplus/Icons.h>
+#include <cppeditor/compilationdb.h>
 #include <cppeditor/cppcodemodelsettings.h>
-#include <cppeditor/cppeditorconstants.h>
 #include <cppeditor/cppeditorwidget.h>
 #include <cppeditor/cppmodelmanager.h>
 #include <cppeditor/cpprefactoringchanges.h>
 #include <cppeditor/cppsemanticinfo.h>
 #include <cppeditor/cpptoolsreuse.h>
-#include <cppeditor/cppvirtualfunctionassistprovider.h>
-#include <cppeditor/cppvirtualfunctionproposalitem.h>
 #include <cppeditor/semantichighlighter.h>
 #include <languageclient/diagnosticmanager.h>
 #include <languageclient/languageclienthoverhandler.h>
@@ -43,25 +40,22 @@
 #include <languageserverprotocol/clientcapabilities.h>
 #include <languageserverprotocol/progresssupport.h>
 #include <projectexplorer/buildconfiguration.h>
+#include <projectexplorer/devicesupport/devicekitaspects.h>
 #include <projectexplorer/devicesupport/idevice.h>
-#include <projectexplorer/kitaspects.h>
+#include <projectexplorer/devicesupport/devicekitaspects.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/target.h>
-#include <projectexplorer/taskhub.h>
-#include <texteditor/codeassist/assistinterface.h>
-#include <texteditor/codeassist/iassistprocessor.h>
-#include <texteditor/codeassist/iassistprovider.h>
-#include <texteditor/codeassist/textdocumentmanipulatorinterface.h>
 #include <texteditor/texteditor.h>
 #include <utils/algorithm.h>
 #include <utils/async.h>
+#include <utils/clangutils.h>
 #include <utils/environment.h>
 #include <utils/fileutils.h>
-#include <utils/itemviews.h>
+#include <utils/mimeconstants.h>
 #include <utils/theme/theme.h>
-#include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QElapsedTimer>
@@ -72,9 +66,7 @@
 #include <QRegularExpression>
 
 #include <cmath>
-#include <new>
 #include <optional>
-#include <set>
 #include <unordered_map>
 #include <utility>
 
@@ -89,6 +81,8 @@ using namespace Utils;
 namespace ClangCodeModel {
 namespace Internal {
 
+using Key = LanguageServerProtocol::Key;
+
 Q_LOGGING_CATEGORY(clangdLog, "qtc.clangcodemodel.clangd", QtWarningMsg);
 Q_LOGGING_CATEGORY(clangdLogAst, "qtc.clangcodemodel.clangd.ast", QtWarningMsg);
 static Q_LOGGING_CATEGORY(clangdLogServer, "qtc.clangcodemodel.clangd.server", QtWarningMsg);
@@ -99,7 +93,7 @@ class SymbolDetails : public JsonObject
 public:
     using JsonObject::JsonObject;
 
-    static constexpr char usrKey[] = "usr";
+    static constexpr Key usrKey{"usr"};
 
     // the unqualified name of the symbol
     QString name() const { return typedValue<QString>(nameKey); }
@@ -134,16 +128,22 @@ class ClangdOutlineItem : public LanguageClientOutlineItem
 private:
     QVariant data(int column, int role) const override
     {
-        switch (role) {
-        case Qt::DisplayRole:
-            return ClangdClient::displayNameFromDocumentSymbol(
-                static_cast<SymbolKind>(type()), name(), detail());
-        case Qt::ForegroundRole:
-            if ((detail().endsWith("class") || detail().endsWith("struct"))
-                && range().end() == selectionRange().end()) {
-                return creatorTheme()->color(Theme::TextColorDisabled);
+        if (valid()) {
+            switch (role) {
+            case Qt::DisplayRole:
+                return ClangdClient::displayNameFromDocumentSymbol(
+                    static_cast<SymbolKind>(type()), name(), detail());
+            case Qt::ForegroundRole:
+                if ((detail().endsWith("class") || detail().endsWith("struct"))
+                    && range().end() == selectionRange().end()) {
+                    return creatorColor(Theme::TextColorDisabled);
+                }
+                break;
+            case AnnotationRole:
+                // Item details are integrated into the displayname through the DisplayRole
+                return {};
             }
-            break;
+
         }
         return LanguageClientOutlineItem::data(column, role);
     }
@@ -151,15 +151,15 @@ private:
 
 void setupClangdConfigFile()
 {
-    const Utils::FilePath targetConfigFile = CppEditor::ClangdSettings::clangdUserConfigFilePath();
-    const Utils::FilePath baseDir = targetConfigFile.parentDir();
+    const FilePath targetConfigFile = CppEditor::ClangdSettings::clangdUserConfigFilePath();
+    const FilePath baseDir = targetConfigFile.parentDir();
     baseDir.ensureWritableDir();
-    Utils::FileReader configReader;
+    const Result<QByteArray> contents = targetConfigFile.fileContents();
     const QByteArray firstLine = "# This file was generated by Qt Creator and will be overwritten "
                                  "unless you remove this line.";
-    if (!configReader.fetch(targetConfigFile) || configReader.data().startsWith(firstLine)) {
-        Utils::FileSaver saver(targetConfigFile);
-        saver.write(firstLine + '\n');
+    if (!contents || contents->startsWith(firstLine)) {
+        FileSaver saver(targetConfigFile);
+        saver.write(QByteArray(firstLine + '\n'));
         saver.write("Hover:\n");
         saver.write("  ShowAKA: Yes\n");
         saver.write("Diagnostics:\n");
@@ -168,35 +168,31 @@ void setupClangdConfigFile()
     }
 }
 
-std::optional<Utils::FilePath> clangdExecutableFromBuildDevice(Project *project)
+static std::optional<Utils::FilePath> clangdExecutableFromBuildDevice(Kit *kit)
 {
-    if (!project)
+    if (!kit)
         return std::nullopt;
 
-    if (ProjectExplorer::Target *target = project->activeTarget()) {
-        if (const ProjectExplorer::IDeviceConstPtr buildDevice = BuildDeviceKitAspect::device(
-                target->kit())) {
-            return buildDevice->clangdExecutable();
-        }
-    }
+    if (const IDeviceConstPtr buildDevice = BuildDeviceKitAspect::device(kit))
+        return buildDevice->clangdExecutable();
 
     return std::nullopt;
 }
 
-static BaseClientInterface *clientInterface(Project *project, const Utils::FilePath &jsonDbDir)
+static BaseClientInterface *clientInterface(BuildConfiguration *bc, const Utils::FilePath &jsonDbDir)
 {
     using CppEditor::ClangdSettings;
     QString indexingOption = "--background-index";
-    const ClangdSettings settings(CppEditor::ClangdProjectSettings(project).settings());
+    const ClangdSettings settings(CppEditor::ClangdProjectSettings(bc).settings());
     const ClangdSettings::IndexingPriority indexingPriority = settings.indexingPriority();
     const bool indexingEnabled = indexingPriority != ClangdSettings::IndexingPriority::Off;
     if (!indexingEnabled)
         indexingOption += "=0";
-    CppEditor::ClangdProjectSettings(project).unblockIndexing();
+    CppEditor::ClangdProjectSettings(bc).unblockIndexing();
     const QString headerInsertionOption = QString("--header-insertion=")
             + (settings.autoIncludeHeaders() ? "iwyu" : "never");
     const QString limitResults = QString("--limit-results=%1").arg(settings.completionResults());
-    const Utils::FilePath clangdExePath = clangdExecutableFromBuildDevice(project).value_or(
+    const Utils::FilePath clangdExePath = clangdExecutableFromBuildDevice(bc ? bc->kit() : nullptr).value_or(
         settings.clangdFilePath());
     Utils::CommandLine cmd{clangdExePath,
                            {indexingOption,
@@ -206,12 +202,11 @@ static BaseClientInterface *clientInterface(Project *project, const Utils::FileP
                             "--clang-tidy=0"}};
     if (settings.workerThreadLimit() != 0)
         cmd.addArg("-j=" + QString::number(settings.workerThreadLimit()));
-    if (indexingEnabled && settings.clangdVersion() >= QVersionNumber(15)) {
+    if (indexingEnabled) {
         cmd.addArg("--background-index-priority="
                    + ClangdSettings::priorityToString(indexingPriority));
     }
-    if (settings.clangdVersion() >= QVersionNumber(16))
-        cmd.addArg("--rename-file-limit=0");
+    cmd.addArg("--rename-file-limit=0");
     if (!jsonDbDir.isEmpty())
         cmd.addArg("--compile-commands-dir=" + clangdExePath.withNewMappedPath(jsonDbDir).path());
     if (clangdLogServer().isDebugEnabled())
@@ -223,6 +218,7 @@ static BaseClientInterface *clientInterface(Project *project, const Utils::FileP
     }
     const auto interface = new StdIOClientInterface;
     interface->setCommandLine(cmd);
+    interface->setAllowCoreDumps(qtcEnvironmentVariableIsSet("QTC_CLANGD_CORE_DUMPS"));
     return interface;
 }
 
@@ -230,15 +226,15 @@ class DiagnosticsCapabilities : public JsonObject
 {
 public:
     using JsonObject::JsonObject;
-    void enableCategorySupport() { insert("categorySupport", true); }
-    void enableCodeActionsInline() {insert("codeActionsInline", true);}
+    void enableCategorySupport() { insert(Key{"categorySupport"}, true); }
+    void enableCodeActionsInline() {insert(Key{"codeActionsInline"}, true);}
 };
 
 class InactiveRegionsCapabilities : public JsonObject
 {
 public:
     using JsonObject::JsonObject;
-    void enableInactiveRegionsSupport() { insert("inactiveRegions", true); }
+    void enableInactiveRegionsSupport() { insert(Key{"inactiveRegions"}, true); }
 };
 
 class ClangdTextDocumentClientCapabilities : public TextDocumentClientCapabilities
@@ -247,9 +243,9 @@ public:
     using TextDocumentClientCapabilities::TextDocumentClientCapabilities;
 
     void setPublishDiagnostics(const DiagnosticsCapabilities &caps)
-    { insert("publishDiagnostics", caps); }
+    { insert(Key{"publishDiagnostics"}, caps); }
     void setInactiveRegionsCapabilities(const InactiveRegionsCapabilities &caps)
-    { insert("inactiveRegionsCapabilities", caps); }
+    { insert(Key{"inactiveRegionsCapabilities"}, caps); }
 };
 
 static qint64 getRevision(const TextDocument *doc)
@@ -320,8 +316,9 @@ public:
 class ClangdClient::Private
 {
 public:
-    Private(ClangdClient *q, Project *project)
-        : q(q), settings(CppEditor::ClangdProjectSettings(project).settings()) {}
+    Private(ClangdClient *q, BuildConfiguration *bc)
+        : q(q), settings(CppEditor::ClangdProjectSettings(bc).settings())
+    {}
 
     void findUsages(TextDocument *document, const QTextCursor &cursor,
                     const QString &searchTerm, const std::optional<QString> &replacement,
@@ -347,14 +344,13 @@ public:
 
     ClangdClient * const q;
     const CppEditor::ClangdSettings::Data settings;
-    ClangdFollowSymbol *followSymbol = nullptr;
+    QList<ClangdFollowSymbol *> followSymbolOps;
     ClangdSwitchDeclDef *switchDeclDef = nullptr;
     ClangdFindLocalReferences *findLocalRefs = nullptr;
     std::optional<QVersionNumber> versionNumber;
 
     QHash<TextDocument *, HighlightingData> highlightingData;
     QHash<Utils::FilePath, CppEditor::BaseEditorDocumentParser::Configuration> parserConfigs;
-    QHash<Utils::FilePath, Tasks> issuePaneEntries;
     QHash<Utils::FilePath, int> openedExtraFiles;
 
     VersionedDataCache<const TextDocument *, ClangdAstNode> astCache;
@@ -382,7 +378,7 @@ static void addToCompilationDb(QJsonObject &cdb,
     const QString fileString = sourceFile.path.toUserOutput();
     args.append(fileString);
     QJsonObject value;
-    value.insert("workingDirectory", workingDir.toString());
+    value.insert("workingDirectory", workingDir.path());
     value.insert("compilationCommand", args);
     cdb.insert(fileString, value);
 }
@@ -392,31 +388,32 @@ static void addCompilationDb(QJsonObject &parentObject, const QJsonObject &cdb)
     parentObject.insert("compilationDatabaseChanges", cdb);
 }
 
-ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, const Id &id)
-    : Client(clientInterface(project, jsonDbDir), id), d(new Private(this, project))
+ClangdClient::ClangdClient(BuildConfiguration *bc, const Utils::FilePath &jsonDbDir, const Id &id)
+    : Client(clientInterface(bc, jsonDbDir), id), d(new Private(this, bc))
 {
     setName(Tr::tr("clangd"));
     LanguageFilter langFilter;
-    using namespace CppEditor::Constants;
+    using namespace Utils::Constants;
     langFilter.mimeTypes = QStringList{C_HEADER_MIMETYPE, C_SOURCE_MIMETYPE,
             CPP_HEADER_MIMETYPE, CPP_SOURCE_MIMETYPE, OBJECTIVE_CPP_SOURCE_MIMETYPE,
             OBJECTIVE_C_SOURCE_MIMETYPE, CUDA_SOURCE_MIMETYPE};
     setSupportedLanguage(langFilter);
-    setActivateDocumentAutomatically(true);
     setCompletionAssistProvider(new ClangdCompletionAssistProvider(this));
+    setFunctionHintAssistProvider(new ClangdFunctionHintProvider(this));
     setQuickFixAssistProvider(new ClangdQuickFixProvider(this));
     symbolSupport().setLimitRenamingToProjects(true);
     symbolSupport().setRenameResultsEnhancer([](const SearchResultItems &symbolOccurrencesInCode) {
         return CppEditor::symbolOccurrencesInDeclarationComments(symbolOccurrencesInCode);
     });
-    if (!project) {
+    if (!bc) {
         QJsonObject initOptions;
         const Utils::FilePath includeDir
                 = CppEditor::ClangdSettings(d->settings).clangdIncludePath();
         CppEditor::CompilerOptionsBuilder optionsBuilder = clangOptionsBuilder(
                     *CppEditor::CppModelManager::fallbackProjectPart(),
                     warningsConfigForProject(nullptr), includeDir, {});
-        const CppEditor::UsePrecompiledHeaders usePch = CppEditor::getPchUsage();
+        const CppEditor::UsePrecompiledHeaders usePch
+            = CppEditor::CppCodeModelSettings::usePrecompiledHeaders(nullptr);
         const QJsonArray projectPartOptions = fullProjectPartOptions(
                     optionsBuilder, globalClangOptions());
         const QJsonArray clangOptions = clangOptionsForFile({}, optionsBuilder.projectPart(),
@@ -430,7 +427,7 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
                && c->state() != Client::Shutdown;
     };
     const QList<Client *> clients =
-        Utils::filtered(LanguageClientManager::clientsForProject(project), isRunningClangdClient);
+            Utils::filtered(LanguageClientManager::clientsForBuildConfiguration(bc), isRunningClangdClient);
     QTC_CHECK(clients.isEmpty());
     for (const Client *client : clients)
         qCWarning(clangdLog) << client->name() << client->stateString();
@@ -450,21 +447,31 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
                 = textCaps->completion();
         if (completionCaps)
             clangdTextCaps.setCompletion(ClangdCompletionCapabilities(*completionCaps));
+
+        // https://clangd.llvm.org/extensions#reference-container
+        if (const auto references = textCaps->references()) {
+            QJsonObject obj = *references;
+            obj.insert("container", true);
+            clangdTextCaps.setReferences(DynamicRegistrationCapabilities(obj));
+        }
+
         caps.setTextDocument(clangdTextCaps);
     }
     caps.clearExperimental();
     setClientCapabilities(caps);
     setLocatorsEnabled(false);
     setAutoRequestCodeActions(false); // clangd sends code actions inside diagnostics
-    progressManager()->setTitleForToken(
-        indexingToken(), project ? Tr::tr("Indexing %1 with clangd").arg(project->displayName())
-                                 : Tr::tr("Indexing session with clangd"));
-    progressManager()->setCancelHandlerForToken(indexingToken(), [this, project]() {
-        CppEditor::ClangdProjectSettings projectSettings(project);
+    progressManager()->setTitleForToken(indexingToken(),
+            bc ? Tr::tr("Indexing %1 with clangd").arg(bc->project()->displayName())
+               : Tr::tr("Indexing session with clangd"));
+    progressManager()->setCancelHandlerForToken(indexingToken(), [this, bc = QPointer(bc)] {
+        if (!bc)
+            return;
+        CppEditor::ClangdProjectSettings projectSettings(bc);
         projectSettings.blockIndexing();
         progressManager()->endProgressReport(indexingToken());
     });
-    setCurrentProject(project);
+    setCurrentBuildConfiguration(bc);
     setDocumentChangeUpdateThreshold(d->settings.documentUpdateThreshold);
     setSemanticTokensHandler([this](TextDocument *doc, const QList<ExpandedSemanticToken> &tokens,
                                     int version, bool force) {
@@ -476,17 +483,18 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
     });
     registerCustomMethod(inactiveRegionsMethodName(), [this](const JsonRpcMessage &msg) {
         handleInactiveRegions(this, msg);
+        return true;
     });
 
     connect(this, &Client::workDone, this,
-            [this, p = QPointer(project)](const ProgressToken &token) {
+            [this, bc = QPointer(bc)](const ProgressToken &token) {
         const QString * const val = std::get_if<QString>(&token);
         if (val && *val == indexingToken()) {
             d->isFullyIndexed = true;
             emit indexingFinished();
 #ifdef WITH_TESTS
-            if (p)
-                emit p->indexingFinished("Indexer.Clangd");
+            if (bc)
+                emit bc->project()->indexingFinished("Indexer.Clangd");
 #endif
         }
     });
@@ -498,8 +506,8 @@ ClangdClient::ClangdClient(Project *project, const Utils::FilePath &jsonDbDir, c
 
 ClangdClient::~ClangdClient()
 {
-    if (d->followSymbol)
-        d->followSymbol->clear();
+    for (ClangdFollowSymbol * const followSymbol : std::as_const(d->followSymbolOps))
+        followSymbol->clear();
     delete d;
 }
 
@@ -517,13 +525,20 @@ void ClangdClient::openExtraFile(const Utils::FilePath &filePath, const QString
         return;
     }
 
-    QFile cxxFile(filePath.toString());
-    if (content.isEmpty() && !cxxFile.open(QIODevice::ReadOnly))
-        return;
+    QString text;
+    if (!content.isEmpty()) {
+        text = content;
+    } else {
+        Result<QByteArray> fileContent = filePath.fileContents();
+        if (!fileContent)
+            return;
+        text = QString::fromUtf8(*std::move(fileContent));
+    }
+
     TextDocumentItem item;
     item.setLanguageId("cpp");
     item.setUri(hostPathToServerUri(filePath));
-    item.setText(!content.isEmpty() ? content : QString::fromUtf8(cxxFile.readAll()));
+    item.setText(std::move(text));
     item.setVersion(0);
     sendMessage(DidOpenTextDocumentNotification(DidOpenTextDocumentParams(item)),
                 SendDocUpdates::Ignore);
@@ -554,9 +569,7 @@ void ClangdClient::findUsages(const CppEditor::CursorInEditor &cursor,
     if (searchTerm.isEmpty())
         return;
 
-    if (replacement && versionNumber() >= QVersionNumber(16)
-        && Utils::qtcEnvironmentVariable("QTC_CLANGD_RENAMING") != "0") {
-
+    if (replacement && Utils::qtcEnvironmentVariable("QTC_CLANGD_RENAMING") != "0") {
         // If we have up-to-date highlighting data, we can prevent giving clangd
         // macros or namespaces to rename, which it can't cope with.
         // TODO: Fix this upstream for macros; see https://github.com/clangd/clangd/issues/729.
@@ -597,7 +610,7 @@ void ClangdClient::findUsages(const CppEditor::CursorInEditor &cursor,
         }
     }
 
-    const bool categorize = CppEditor::codeModelSettings()->categorizeFindReferences();
+    const bool categorize = CppEditor::CppCodeModelSettings::categorizeFindReferences();
 
     // If it's a "normal" symbol, go right ahead.
     if (searchTerm != "operator" && Utils::allOf(searchTerm, [](const QChar &c) {
@@ -686,30 +699,14 @@ const LanguageClient::Client::CustomInspectorTabs ClangdClient::createCustomInsp
 
 class ClangdDiagnosticManager : public LanguageClient::DiagnosticManager
 {
-    using LanguageClient::DiagnosticManager::DiagnosticManager;
-
-    ClangdClient *getClient() const { return qobject_cast<ClangdClient *>(client()); }
-
-    bool isCurrentDocument(const Utils::FilePath &filePath) const
+public:
+    ClangdDiagnosticManager(LanguageClient::Client *client)
+        : LanguageClient::DiagnosticManager(client)
     {
-        const IDocument * const doc = EditorManager::currentDocument();
-        return doc && doc->filePath() == filePath;
-    }
-
-    void showDiagnostics(const Utils::FilePath &filePath, int version) override
-    {
-        getClient()->clearTasks(filePath);
-        DiagnosticManager::showDiagnostics(filePath, version);
-        if (isCurrentDocument(filePath))
-            getClient()->switchIssuePaneEntries(filePath);
-    }
-
-    void hideDiagnostics(const Utils::FilePath &filePath) override
-    {
-        DiagnosticManager::hideDiagnostics(filePath);
-        if (isCurrentDocument(filePath))
-            TaskHub::clearTasks(Constants::TASK_CATEGORY_DIAGNOSTICS);
+        setTaskCategory(Constants::TASK_CATEGORY_DIAGNOSTICS);
+        setForceCreateTasks(false);
     }
+private:
 
     QList<Diagnostic> filteredDiagnostics(const QList<Diagnostic> &diagnostics) const override
     {
@@ -725,7 +722,18 @@ class ClangdDiagnosticManager : public LanguageClient::DiagnosticManager
                              const Diagnostic &diagnostic,
                              bool isProjectFile) const override
     {
-        return new ClangdTextMark(doc, diagnostic, isProjectFile, getClient());
+        return new ClangdTextMark(
+                    doc, diagnostic, isProjectFile, qobject_cast<ClangdClient *>(client()));
+    }
+
+    QString taskText(const Diagnostic &diagnostic) const override
+    {
+        QString text = diagnostic.message();
+        auto splitIndex = text.indexOf("\n\n");
+        if (splitIndex >= 0)
+            text.truncate(splitIndex);
+
+        return diagnosticCategoryPrefixRemoved(text);
     }
 };
 
@@ -769,10 +777,18 @@ QList<Text::Range> ClangdClient::additionalDocumentHighlights(
         qobject_cast<CppEditor::CppEditorWidget *>(editorWidget), cursor);
 }
 
-RefactoringChangesData *ClangdClient::createRefactoringChangesBackend() const
+bool ClangdClient::shouldSendDidSave(const TextEditor::TextDocument *doc) const
 {
-    return new CppEditor::CppRefactoringChangesData(
-                CppEditor::CppModelManager::snapshot());
+    for (const Project * const p : ProjectManager::projects()) {
+        if (const Node * const n  = p->nodeForFilePath(doc->filePath()))
+            return n->asFileNode() && n->asFileNode()->fileType() == FileType::Header;
+    }
+    return CppEditor::ProjectFile::isHeader(doc->filePath());
+}
+
+RefactoringFilePtr ClangdClient::createRefactoringFile(const FilePath &filePath) const
+{
+    return CppEditor::CppRefactoringChanges(CppEditor::CppModelManager::snapshot()).file(filePath);
 }
 
 QVersionNumber ClangdClient::versionNumber() const
@@ -780,7 +796,7 @@ QVersionNumber ClangdClient::versionNumber() const
     if (d->versionNumber)
         return d->versionNumber.value();
 
-    const QRegularExpression versionPattern("^clangd version (\\d+)\\.(\\d+)\\.(\\d+).*$");
+    static const QRegularExpression versionPattern("^clangd version (\\d+)\\.(\\d+)\\.(\\d+).*$");
     QTC_CHECK(versionPattern.isValid());
     const QRegularExpressionMatch match = versionPattern.match(serverVersion());
     if (match.isValid()) {
@@ -914,12 +930,18 @@ void ClangdClient::updateParserConfig(const Utils::FilePath &filePath,
     CppEditor::CompilerOptionsBuilder optionsBuilder = clangOptionsBuilder(
                 *projectPart, warningsConfigForProject(project()), includeDir,
                 ProjectExplorer::Macro::toMacros(config.editorDefines));
-    const CppEditor::ProjectFile file(filePath,
-                                      CppEditor::ProjectFile::classify(filePath.toString()));
+    const CppEditor::ProjectFile file(filePath, CppEditor::ProjectFile::classify(filePath));
     const QJsonArray projectPartOptions = fullProjectPartOptions(
                 optionsBuilder, globalClangOptions());
-    addToCompilationDb(cdbChanges, *projectPart, CppEditor::getPchUsage(), projectPartOptions,
-                       filePath.parentDir(), file, optionsBuilder.isClStyle());
+    const auto cppSettings = CppEditor::CppCodeModelSettings::settingsForProject(
+        projectPart->topLevelProject);
+    addToCompilationDb(cdbChanges,
+                       *projectPart,
+                       cppSettings.usePrecompiledHeaders(),
+                       projectPartOptions,
+                       filePath.parentDir(),
+                       file,
+                       optionsBuilder.isClStyle());
     QJsonObject settings;
     addCompilationDb(settings, cdbChanges);
     DidChangeConfigurationParams configChangeParams;
@@ -928,24 +950,6 @@ void ClangdClient::updateParserConfig(const Utils::FilePath &filePath,
     emit configChanged();
 }
 
-void ClangdClient::switchIssuePaneEntries(const FilePath &filePath)
-{
-    TaskHub::clearTasks(Constants::TASK_CATEGORY_DIAGNOSTICS);
-    const Tasks tasks = d->issuePaneEntries.value(filePath);
-    for (const Task &t : tasks)
-        TaskHub::addTask(t);
-}
-
-void ClangdClient::addTask(const ProjectExplorer::Task &task)
-{
-    d->issuePaneEntries[task.file] << task;
-}
-
-void ClangdClient::clearTasks(const Utils::FilePath &filePath)
-{
-    d->issuePaneEntries[filePath].clear();
-}
-
 std::optional<bool> ClangdClient::hasVirtualFunctionAt(TextDocument *doc, int revision,
                                                          const Range &range)
 {
@@ -981,7 +985,7 @@ MessageId ClangdClient::requestSymbolInfo(const Utils::FilePath &filePath, const
         // According to the documentation, we should receive a single
         // object here, but it's a list. No idea what it means if there's
         // more than one entry. We choose the first one.
-        const auto list = std::get_if<QList<SymbolDetails>>(&result.value());
+        const auto list = std::get_if<QList<SymbolDetails>>(&(*result));
         if (!list || list->isEmpty()) {
             handler({}, {}, reqId);
             return;
@@ -997,7 +1001,7 @@ MessageId ClangdClient::requestSymbolInfo(const Utils::FilePath &filePath, const
 #ifdef WITH_TESTS
 ClangdFollowSymbol *ClangdClient::currentFollowSymbolOperation()
 {
-    return d->followSymbol;
+    return d->followSymbolOps.isEmpty() ? nullptr : d->followSymbolOps.first();
 }
 #endif
 
@@ -1012,8 +1016,21 @@ void ClangdClient::followSymbol(TextDocument *document,
 {
     QTC_ASSERT(documentOpen(document), openDocument(document));
 
-    if (d->followSymbol)
-        d->followSymbol->cancel();
+    const ClangdFollowSymbol::Origin origin
+        = CppEditor::CppCodeModelSettings::isInteractiveFollowSymbol()
+              ? ClangdFollowSymbol::Origin::User
+              : ClangdFollowSymbol::Origin::Code;
+    if (origin == ClangdFollowSymbol::Origin::User) {
+        for (auto it = d->followSymbolOps.begin(); it != d->followSymbolOps.end(); ) {
+            ClangdFollowSymbol * const followSymbol = *it;
+            if (followSymbol->isInteractive()) {
+                it = d->followSymbolOps.erase(it);
+                followSymbol->cancel();
+            } else {
+                ++it;
+            }
+        }
+    }
 
     const QTextCursor adjustedCursor = d->adjustedCursor(cursor, document);
     if (followTo == FollowTo::SymbolDef && !resolveTarget) {
@@ -1027,14 +1044,13 @@ void ClangdClient::followSymbol(TextDocument *document,
 
     qCDebug(clangdLog) << "follow symbol requested" << document->filePath()
                        << adjustedCursor.blockNumber() << adjustedCursor.positionInBlock();
-    auto clangdFollowSymbol = new ClangdFollowSymbol(this, adjustedCursor, editorWidget, document,
-                                                     callback, followTo, openInSplit);
+    auto clangdFollowSymbol = new ClangdFollowSymbol(this, origin, adjustedCursor, editorWidget,
+                                                     document, callback, followTo, openInSplit);
     connect(clangdFollowSymbol, &ClangdFollowSymbol::done, this, [this, clangdFollowSymbol] {
         clangdFollowSymbol->deleteLater();
-        if (clangdFollowSymbol == d->followSymbol)
-            d->followSymbol = nullptr;
+        d->followSymbolOps.removeOne(clangdFollowSymbol);
     });
-    d->followSymbol = clangdFollowSymbol;
+    d->followSymbolOps << clangdFollowSymbol;
 }
 
 void ClangdClient::switchDeclDef(TextDocument *document, const QTextCursor &cursor,
@@ -1139,14 +1155,15 @@ void ClangdClient::gatherHelpItemForTooltip(const HoverRequest::Response &hoverR
                 for (const QString &line : lines) {
                     const QString possibleFilePath = line.simplified();
                     const auto looksLikeFilePath = [&] {
-                        if (possibleFilePath.length() < 3)
+                        if (possibleFilePath.length() < 4)
                             return false;
                         if (osType() == OsTypeWindows) {
-                            if (possibleFilePath.startsWith(R"(\\)"))
+                            if (possibleFilePath.startsWith(R"(\\\\)"))
                                 return true;
                             return possibleFilePath.front().isLetter()
                                     && possibleFilePath.at(1) == ':'
-                                    && possibleFilePath.at(2) == '\\';
+                                    && possibleFilePath.at(2) == '\\'
+                                    && possibleFilePath.at(3) == '\\';
                         }
                         return possibleFilePath.front() == '/'
                                 && possibleFilePath.at(1).isLetterOrNumber();
@@ -1459,7 +1476,7 @@ QTextCursor ClangdClient::Private::adjustedCursor(const QTextCursor &cursor,
             return c;
         }
 
-        // QVector<QString|>
+        // QList<QString|>
         if (const TemplateIdAST * const templAst = (*it)->asTemplateId()) {
             if (posForToken(templAst->greater_token) == cursor.position())
                 return leftMovedCursor();
@@ -1495,10 +1512,9 @@ void ClangdClient::Private::setHelpItemForTooltip(const MessageId &token,
         mark = type;
 
     const HelpItem helpItem(helpIds, filePath, mark, category);
+    q->hoverHandler()->setHelpItem(token, helpItem);
     if (isTesting)
-        emit q->helpItemGathered(helpItem);
-    else
-        q->hoverHandler()->setHelpItem(token, helpItem);
+        emit q->helpItemGathered(helpItem, q->hoverHandler()->toolTip());
 }
 
 // Unfortunately, clangd ignores almost everything except symbols when sending
@@ -1526,7 +1542,7 @@ void ClangdClient::Private::handleSemanticTokens(TextDocument *doc,
         return;
     }
     force = force || isTesting;
-    const auto data = highlightingData.find(doc);
+    auto data = highlightingData.find(doc);
     if (data != highlightingData.end()) {
         if (!force && data->previousTokens.first == tokens
                 && data->previousTokens.second == version) {
@@ -1536,66 +1552,55 @@ void ClangdClient::Private::handleSemanticTokens(TextDocument *doc,
         data->previousTokens.first = tokens;
         data->previousTokens.second = version;
     } else {
-        highlightingData.insert(doc, {{tokens, version}, {}});
+        data = highlightingData.insert(doc, {{tokens, version}, {}});
     }
     for (const ExpandedSemanticToken &t : tokens)
         qCDebug(clangdLogHighlight()) << '\t' << t.line << t.column << t.length << t.type
                                       << t.modifiers;
 
-    const auto astHandler = [this, tokens, doc, version](const ClangdAstNode &ast, const MessageId &) {
-        FinalizingSubtaskTimer t(highlightingTimer);
-        if (!q->documentOpen(doc))
-            return;
-        if (version != q->documentVersion(doc->filePath())) {
-            qCInfo(clangdLogHighlight) << "AST not up to date; aborting highlighting procedure"
-                                        << version << q->documentVersion(doc->filePath());
-            return;
+    FinalizingSubtaskTimer ft(highlightingTimer);
+    if (!q->documentOpen(doc))
+        return;
+    if (version != q->documentVersion(doc->filePath())) {
+        qCInfo(clangdLogHighlight) << "AST not up to date; aborting highlighting procedure"
+                                   << version << q->documentVersion(doc->filePath());
+        return;
+    }
+
+    const auto runner = [tokens, filePath = doc->filePath(),
+                         text = doc->plainText(),
+                         rev = doc->document()->revision(), this] {
+        try {
+            return Utils::asyncRun(doSemanticHighlighting, filePath, tokens, text,
+                                   rev, highlightingTimer);
+        } catch (const std::exception &e) {
+            qWarning() << "caught" << e.what() << "in main highlighting thread";
+            return QFuture<HighlightingResult>();
         }
-        if (clangdLogAst().isDebugEnabled())
-            ast.print();
-
-        const auto runner = [tokens, filePath = doc->filePath(),
-                             text = doc->document()->toPlainText(), ast,
-                             doc = QPointer(doc), rev = doc->document()->revision(),
-                             clangdVersion = q->versionNumber(),
-                             this] {
-            try {
-                return Utils::asyncRun(doSemanticHighlighting, filePath, tokens, text, ast, doc,
-                                       rev, clangdVersion, highlightingTimer);
-            } catch (const std::exception &e) {
-                qWarning() << "caught" << e.what() << "in main highlighting thread";
-                return QFuture<HighlightingResult>();
-            }
-        };
-
-        if (isTesting) {
-            const auto watcher = new QFutureWatcher<HighlightingResult>(q);
-            connect(watcher, &QFutureWatcher<HighlightingResult>::finished,
-                    q, [this, watcher, fp = doc->filePath()] {
-                emit q->highlightingResultsReady(watcher->future().results(), fp);
-                watcher->deleteLater();
-            });
-            watcher->setFuture(runner());
-            return;
-        }
-
-        auto &data = highlightingData[doc];
-        if (!data.highlighter)
-            data.highlighter = new CppEditor::SemanticHighlighter(doc);
-        else
-            data.highlighter->updateFormatMapFromFontSettings();
-        data.highlighter->setHighlightingRunner(runner);
-        data.highlighter->run();
     };
-    if (q->versionNumber().majorVersion() >= 17)
-        astHandler({}, {});
+
+    if (isTesting) {
+        const auto watcher = new QFutureWatcher<HighlightingResult>(q);
+        connect(watcher, &QFutureWatcher<HighlightingResult>::finished,
+                q, [this, watcher, fp = doc->filePath()] {
+            emit q->highlightingResultsReady(watcher->future().results(), fp);
+            watcher->deleteLater();
+        });
+        watcher->setFuture(runner());
+        return;
+    }
+
+    if (!data->highlighter)
+        data->highlighter = new CppEditor::SemanticHighlighter(doc);
     else
-        getAndHandleAst(doc, astHandler, AstCallbackMode::SyncIfPossible);
+        data->highlighter->updateFormatMapFromFontSettings();
+    data->highlighter->setHighlightingRunner(runner);
+    data->highlighter->run();
 }
 
 std::optional<QList<CodeAction> > ClangdDiagnostic::codeActions() const
 {
-    auto actions = optionalArray<LanguageServerProtocol::CodeAction>("codeActions");
+    auto actions = optionalArray<LanguageServerProtocol::CodeAction>(Key{"codeActions"});
     if (!actions)
         return actions;
     static const QStringList badCodeActions{
@@ -1612,7 +1617,7 @@ std::optional<QList<CodeAction> > ClangdDiagnostic::codeActions() const
 
 QString ClangdDiagnostic::category() const
 {
-    return typedValue<QString>("category");
+    return typedValue<QString>(Key{"category"});
 }
 
 MessageId ClangdClient::Private::getAndHandleAst(const TextDocOrFile &doc,
