@@ -68,9 +68,7 @@ QSize FancyTabBar::tabSizeHint(bool minimum) const
                     Core::Constants::MODEBAR_ICONSONLY_BUTTON_SIZE / (minimum ? 3 : 1)};
     }
 
-    QFont boldFont(font());
-    boldFont.setPointSizeF(StyleHelper::sidebarFontSize());
-    boldFont.setBold(true);
+    const QFont boldFont = StyleHelper::uiFont(StyleHelper::UiElementCaptionStrong);
     const QFontMetrics fm(boldFont);
     const int spacing = 8;
     const int width = 60 + spacing + 2;
@@ -93,25 +91,37 @@ void FancyTabBar::paintEvent(QPaintEvent *event)
         p.fillRect(event->rect(), StyleHelper::baseColor());
     }
 
-    for (int i = 0; i < count(); ++i)
+    int visibleIndex = 0;
+    int visibleCurrentIndex = -1;
+    for (int i = 0; i < count(); ++i) {
+        if (!m_tabs.at(i)->visible)
+            continue;
         if (i != currentIndex())
-            paintTab(&p, i);
+            paintTab(&p, i, visibleIndex, QIcon::Off);
+        else
+            visibleCurrentIndex = visibleIndex;
+        ++visibleIndex;
+    }
 
     // paint active tab last, since it overlaps the neighbors
     if (currentIndex() != -1)
-        paintTab(&p, currentIndex());
+        paintTab(&p, currentIndex(), visibleCurrentIndex, QIcon::On);
 }
 
 // Handle hover events for mouse fade ins
 void FancyTabBar::mouseMoveEvent(QMouseEvent *event)
 {
     int newHover = -1;
+    int visibleIndex = 0;
     for (int i = 0; i < count(); ++i) {
-        const QRect area = tabRect(i);
+        if (!m_tabs.at(i)->visible)
+            continue;
+        const QRect area = tabRect(visibleIndex);
         if (area.contains(event->pos())) {
             newHover = i;
             break;
         }
+        ++visibleIndex;
     }
     if (newHover == m_hoverIndex)
         return;
@@ -123,7 +133,7 @@ void FancyTabBar::mouseMoveEvent(QMouseEvent *event)
 
     if (validIndex(m_hoverIndex)) {
         m_tabs[m_hoverIndex]->fadeIn();
-        m_hoverRect = tabRect(m_hoverIndex);
+        m_hoverRect = tabRect(visibleIndex);
     }
 }
 
@@ -171,43 +181,60 @@ QSize FancyTabBar::minimumSizeHint() const
     return {sh.width(), sh.height() * int(m_tabs.count())};
 }
 
-QRect FancyTabBar::tabRect(int index) const
+QRect FancyTabBar::tabRect(int visibleIndex) const
 {
     QSize sh = tabSizeHint();
 
     if (sh.height() * m_tabs.count() > height())
         sh.setHeight(height() / m_tabs.count());
 
-    return {0, index * sh.height(), sh.width(), sh.height()};
+    return {0, visibleIndex * sh.height(), sh.width(), sh.height()};
+}
+
+int FancyTabBar::visibleIndex(int index) const
+{
+    int vIndex = 0;
+    for (int i = 0; i < m_tabs.size(); ++i) {
+        if (i == index)
+            return vIndex;
+        if (m_tabs.at(i)->visible)
+            ++vIndex;
+    }
+    return vIndex;
 }
 
 void FancyTabBar::mousePressEvent(QMouseEvent *event)
 {
     event->accept();
+    int visibleIndex = 0;
     for (int index = 0; index < m_tabs.count(); ++index) {
-        const QRect rect = tabRect(index);
+        if (!m_tabs.at(index)->visible)
+            continue;
+        const QRect rect = tabRect(visibleIndex);
         if (rect.contains(event->pos())) {
-            if (isTabEnabled(index)) {
+            if (isTabEnabled(index) && event->button() == Qt::LeftButton) {
                 if (m_tabs.at(index)->hasMenu
-                    && ((!m_iconsOnly && rect.right() - event->pos().x() <= kMenuButtonWidth)
-                        || event->button() == Qt::RightButton)) {
-                    // menu arrow clicked or right-click
+                    && (!m_iconsOnly && rect.right() - event->pos().x() <= kMenuButtonWidth)) {
+                    // menu arrow clicked
                     emit menuTriggered(index, event);
                 } else {
                     if (index != m_currentIndex) {
                         emit currentAboutToChange(index);
                         m_currentIndex = index;
                         update();
-                        // update tab bar before showing widget
-                        QMetaObject::invokeMethod(this, [this] {
-                            emit currentChanged(m_currentIndex);
-                        }, Qt::QueuedConnection);
+                        emit currentChanged(m_currentIndex);
                     }
                 }
+            } else if (event->button() == Qt::RightButton) {
+                emit menuTriggered(index, event);
             }
-            break;
+            return;
         }
+        ++visibleIndex;
     }
+    // not in a mode button
+    if (event->button() == Qt::RightButton)
+        emit menuTriggered(-1, event);
 }
 
 static void paintSelectedTabBackground(QPainter *painter, const QRect &spanRect)
@@ -260,10 +287,18 @@ static void paintSelectedTabBackground(QPainter *painter, const QRect &spanRect)
     painter->drawPixmap(spanRect.topLeft() + QPoint(0, -verticalOverlap), selection);
 }
 
+static void paintHighlight(QPainter *painter, const QRect &rect)
+{
+    QRect accentRect = rect;
+    accentRect.setWidth(2);
+    painter->fillRect(accentRect, creatorColor(Theme::FancyToolButtonHighlightColor));
+}
+
 static void paintIcon(QPainter *painter, const QRect &rect,
-                      const QIcon &icon,
+                      const QIcon &icon, QIcon::State iconState,
                       bool enabled, bool selected)
 {
+    painter->save();
     const QIcon::Mode iconMode = enabled ? (selected ? QIcon::Active : QIcon::Normal)
                                          : QIcon::Disabled;
     QRect iconRect(0, 0, Core::Constants::MODEBAR_ICON_SIZE, Core::Constants::MODEBAR_ICON_SIZE);
@@ -271,23 +306,16 @@ static void paintIcon(QPainter *painter, const QRect &rect,
     iconRect = iconRect.intersected(rect);
     if (!enabled && !creatorTheme()->flag(Theme::FlatToolBars))
         painter->setOpacity(0.7);
-    StyleHelper::drawIconWithShadow(icon, iconRect, painter, iconMode);
-
-    if (selected && creatorTheme()->flag(Theme::FlatToolBars)) {
-        painter->setOpacity(1.0);
-        QRect accentRect = rect;
-        accentRect.setWidth(2);
-        painter->fillRect(accentRect, creatorTheme()->color(Theme::IconsBaseColor));
-    }
+    StyleHelper::drawIconWithShadow(icon, iconRect, painter, iconMode, iconState);
+    painter->restore();
 }
 
 static void paintIconAndText(QPainter *painter, const QRect &rect,
-                             const QIcon &icon, const QString &text,
+                             const QIcon &icon, QIcon::State iconState, const QString &text,
                              bool enabled, bool selected)
 {
-    QFont boldFont(painter->font());
-    boldFont.setPointSizeF(StyleHelper::sidebarFontSize());
-    boldFont.setBold(true);
+    painter->save();
+    const QFont boldFont = StyleHelper::uiFont(StyleHelper::UiElementCaptionStrong);
     painter->setFont(boldFont);
 
     const bool drawIcon = rect.height() > 36;
@@ -302,23 +330,18 @@ static void paintIconAndText(QPainter *painter, const QRect &rect,
         iconRect = iconRect.intersected(tabIconRect);
         if (!enabled && !creatorTheme()->flag(Theme::FlatToolBars))
             painter->setOpacity(0.7);
-        StyleHelper::drawIconWithShadow(icon, iconRect, painter, iconMode);
+        StyleHelper::drawIconWithShadow(icon, iconRect, painter, iconMode, iconState);
     }
 
     painter->setOpacity(1.0); //FIXME: was 0.7 before?
-    if (selected && creatorTheme()->flag(Theme::FlatToolBars)) {
-        QRect accentRect = rect;
-        accentRect.setWidth(2);
-        painter->fillRect(accentRect, creatorTheme()->color(Theme::IconsBaseColor));
-    }
     if (enabled) {
         painter->setPen(
-            selected ? creatorTheme()->color(Theme::FancyTabWidgetEnabledSelectedTextColor)
-                     : creatorTheme()->color(Theme::FancyTabWidgetEnabledUnselectedTextColor));
+            selected ? creatorColor(Theme::FancyTabWidgetEnabledSelectedTextColor)
+                     : creatorColor(Theme::FancyTabWidgetEnabledUnselectedTextColor));
     } else {
         painter->setPen(
-            selected ? creatorTheme()->color(Theme::FancyTabWidgetDisabledSelectedTextColor)
-                     : creatorTheme()->color(Theme::FancyTabWidgetDisabledUnselectedTextColor));
+            selected ? creatorColor(Theme::FancyTabWidgetDisabledSelectedTextColor)
+                     : creatorColor(Theme::FancyTabWidgetDisabledUnselectedTextColor));
     }
 
     painter->translate(0, -1);
@@ -327,9 +350,11 @@ static void paintIconAndText(QPainter *painter, const QRect &rect,
     const int textFlags = Qt::AlignCenter | (drawIcon ? Qt::AlignBottom : Qt::AlignVCenter)
                           | Qt::TextWordWrap;
     painter->drawText(tabTextRect, textFlags, text);
+    painter->restore();
 }
 
-void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
+void FancyTabBar::paintTab(QPainter *painter, int tabIndex, int visibleIndex,
+                           QIcon::State iconState) const
 {
     if (!validIndex(tabIndex)) {
         qWarning("invalid index");
@@ -338,14 +363,14 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
     painter->save();
 
     const FancyTab *tab = m_tabs.at(tabIndex);
-    const QRect rect = tabRect(tabIndex);
+    const QRect rect = tabRect(visibleIndex);
     const bool selected = (tabIndex == m_currentIndex);
     const bool enabled = isTabEnabled(tabIndex);
 
     if (selected) {
         if (creatorTheme()->flag(Theme::FlatToolBars)) {
             // background color of a fancy tab that is active
-            painter->fillRect(rect, creatorTheme()->color(Theme::FancyTabBarSelectedBackgroundColor));
+            painter->fillRect(rect, creatorColor(Theme::FancyTabBarSelectedBackgroundColor));
         } else {
             paintSelectedTabBackground(painter, rect);
         }
@@ -356,16 +381,19 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
         painter->save();
         painter->setOpacity(fader);
         if (creatorTheme()->flag(Theme::FlatToolBars))
-            painter->fillRect(rect, creatorTheme()->color(Theme::FancyToolButtonHoverColor));
+            painter->fillRect(rect, creatorColor(Theme::FancyToolButtonHoverColor));
         else
             FancyToolButton::hoverOverlay(painter, rect);
         painter->restore();
     }
 
     if (m_iconsOnly)
-        paintIcon(painter, rect, tab->icon, enabled, selected);
+        paintIcon(painter, rect, tab->icon, iconState, enabled, selected);
     else
-        paintIconAndText(painter, rect, tab->icon, tab->text, enabled, selected);
+        paintIconAndText(painter, rect, tab->icon, iconState, tab->text, enabled, selected);
+
+    if (selected && creatorTheme()->flag(Theme::FlatToolBars))
+        paintHighlight(painter, rect);
 
     // menu arrow
     if (tab->hasMenu && !m_iconsOnly) {
@@ -379,7 +407,7 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
 
 void FancyTabBar::setCurrentIndex(int index)
 {
-    if (isTabEnabled(index) && index != m_currentIndex) {
+    if ((index == -1 || isTabEnabled(index)) && index != m_currentIndex) {
         emit currentAboutToChange(index);
         m_currentIndex = index;
         update();
@@ -400,7 +428,8 @@ void FancyTabBar::setTabEnabled(int index, bool enable)
 
     if (index < m_tabs.size() && index >= 0) {
         m_tabs[index]->enabled = enable;
-        update(tabRect(index));
+        if (m_tabs[index]->visible)
+            update(tabRect(visibleIndex(index)));
     }
 }
 
@@ -415,6 +444,15 @@ bool FancyTabBar::isTabEnabled(int index) const
     return false;
 }
 
+void FancyTabBar::setTabVisible(int index, bool visible)
+{
+    QTC_ASSERT(index < m_tabs.size(), return);
+    QTC_ASSERT(index >= 0, return);
+
+    m_tabs[index]->visible = visible;
+    update();
+}
+
 class FancyColorButton : public QWidget
 {
     Q_OBJECT
@@ -426,7 +464,7 @@ public:
         setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
     }
 
-    void mousePressEvent(QMouseEvent *ev) override { emit clicked(ev->button(), ev->modifiers()); }
+    void mousePressEvent(QMouseEvent *ev) override { emit clicked(ev); }
 
     void paintEvent(QPaintEvent *event) override
     {
@@ -444,7 +482,7 @@ public:
     }
 
 signals:
-    void clicked(Qt::MouseButton button, Qt::KeyboardModifiers modifiers);
+    void clicked(QMouseEvent *ev);
 };
 
 //////
@@ -459,33 +497,40 @@ FancyTabWidget::FancyTabWidget(QWidget *parent)
 
     auto bar = new StyledBar;
     auto fancyButton = new FancyColorButton;
-    connect(fancyButton, &FancyColorButton::clicked, this, &FancyTabWidget::topAreaClicked);
+    connect(fancyButton, &FancyColorButton::clicked, this, [this](QMouseEvent *event) {
+        if (event->button() == Qt::RightButton)
+            emit menuTriggered(-1, event);
+        else
+            emit topAreaClicked(event);
+    });
 
     m_modesStack = new QStackedLayout;
+    m_modesStack->addWidget(new QWidget(this));
     m_statusBar = new QStatusBar;
     m_statusBar->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
 
     QVBoxLayout *vlayout;
 
+    // clang-format off
     using namespace Layouting;
-    Row { fancyButton, noMargin() }.attachTo(bar);
+    Row { fancyButton, noMargin }.attachTo(bar);
     Row {
         Widget {
             bindTo(&m_selectionWidget),
             Column {
                 bar,
                 m_tabBar,
-                st,
                 Widget {
                     bindTo(&m_cornerWidgetContainer),
-                    Column { st, spacing(0), noMargin() },
+                    Column { spacing(0), noMargin },
                 },
-                spacing(0),  noMargin(),
+                spacing(0), noMargin,
             },
         },
         Column { bindTo(&vlayout), m_modesStack, m_statusBar, spacing(0) },
-        spacing(1), noMargin(),
+        spacing(1), noMargin,
     }.attachTo(this);
+    // clang-format on
 
     m_selectionWidget->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
 
@@ -512,13 +557,13 @@ bool FancyTabWidget::isSelectionWidgetVisible() const
 
 void FancyTabWidget::insertTab(int index, QWidget *tab, const QIcon &icon, const QString &label, bool hasMenu)
 {
-    m_modesStack->insertWidget(index, tab);
+    m_modesStack->insertWidget(index + 1, tab);
     m_tabBar->insertTab(index, icon, label, hasMenu);
 }
 
 void FancyTabWidget::removeTab(int index)
 {
-    m_modesStack->removeWidget(m_modesStack->widget(index));
+    m_modesStack->removeWidget(m_modesStack->widget(index + 1));
     m_tabBar->removeTab(index);
 }
 
@@ -582,11 +627,9 @@ QStatusBar *FancyTabWidget::statusBar() const
     return m_statusBar;
 }
 
-InfoBar *FancyTabWidget::infoBar()
+void FancyTabWidget::setInfoBar(InfoBar *infoBar)
 {
-    if (!m_infoBarDisplay.infoBar())
-        m_infoBarDisplay.setInfoBar(&m_infoBar);
-    return &m_infoBar;
+    m_infoBarDisplay.setInfoBar(infoBar);
 }
 
 void FancyTabWidget::setCurrentIndex(int index)
@@ -596,7 +639,7 @@ void FancyTabWidget::setCurrentIndex(int index)
 
 void FancyTabWidget::showWidget(int index)
 {
-    m_modesStack->setCurrentIndex(index);
+    m_modesStack->setCurrentIndex(index + 1);
     QWidget *w = m_modesStack->currentWidget();
     if (QTC_GUARD(w)) {
         if (QWidget *focusWidget = w->focusWidget())
@@ -621,6 +664,11 @@ bool FancyTabWidget::isTabEnabled(int index) const
     return m_tabBar->isTabEnabled(index);
 }
 
+void FancyTabWidget::setTabVisible(int index, bool visible)
+{
+    m_tabBar->setTabVisible(index, visible);
+}
+
 void FancyTabWidget::setIconsOnly(bool iconsOnly)
 {
     m_tabBar->setIconsOnly(iconsOnly);
