@@ -1,38 +1,21 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "propertyeditorcontextobject.h"
-#include "timelineeditor/easingcurvedialog.h"
 
-#include <abstractview.h>
-#include <nodemetainfo.h>
-#include <rewritingexception.h>
-#include <qmldesignerplugin.h>
-#include <qmlmodelnodeproxy.h>
-#include <qmlobjectnode.h>
-#include <qmltimeline.h>
+#include "abstractview.h"
+#include "compatibleproperties.h"
+#include "easingcurvedialog.h"
+#include "nodemetainfo.h"
+#include "propertyeditorutils.h"
+#include "qml3dnode.h"
+#include "qmldesignerconstants.h"
+#include "qmldesignerplugin.h"
+#include "qmlmodelnodeproxy.h"
+#include "qmlobjectnode.h"
+#include "qmltimeline.h"
+
+#include <qmldesignerbase/settings/designersettings.h>
 
 #include <coreplugin/messagebox.h>
 #include <utils/algorithm.h>
@@ -41,71 +24,35 @@
 #include <QApplication>
 #include <QCursor>
 #include <QFontDatabase>
+#include <QLoggingCategory>
 #include <QMessageBox>
 #include <QQmlContext>
+#include <QQuickWidget>
 #include <QWindow>
 
 #include <coreplugin/icore.h>
 
-static uchar fromHex(const uchar c, const uchar c2)
-{
-    uchar rv = 0;
-    if (c >= '0' && c <= '9')
-        rv += (c - '0') * 16;
-    else if (c >= 'A' && c <= 'F')
-        rv += (c - 'A' + 10) * 16;
-    else if (c >= 'a' && c <= 'f')
-        rv += (c - 'a' + 10) * 16;
-
-    if (c2 >= '0' && c2 <= '9')
-        rv += (c2 - '0');
-    else if (c2 >= 'A' && c2 <= 'F')
-        rv += (c2 - 'A' + 10);
-    else if (c2 >= 'a' && c2 <= 'f')
-        rv += (c2 - 'a' + 10);
-
-    return rv;
-}
-
-static uchar fromHex(const QString &s, int idx)
-{
-    uchar c = s.at(idx).toLatin1();
-    uchar c2 = s.at(idx + 1).toLatin1();
-    return fromHex(c, c2);
-}
-
-QColor convertColorFromString(const QString &s)
-{
-    if (s.length() == 9 && s.startsWith(QLatin1Char('#'))) {
-        uchar a = fromHex(s, 1);
-        uchar r = fromHex(s, 3);
-        uchar g = fromHex(s, 5);
-        uchar b = fromHex(s, 7);
-        return {r, g, b, a};
-    } else {
-        QColor rv(s);
-        return rv;
-    }
-}
-
 namespace QmlDesigner {
 
-PropertyEditorContextObject::PropertyEditorContextObject(QObject *parent)
+static Q_LOGGING_CATEGORY(urlSpecifics, "qtc.propertyeditor.specifics", QtWarningMsg)
+
+PropertyEditorContextObject::PropertyEditorContextObject(QQuickWidget *widget, QObject *parent)
     : QObject(parent)
     , m_isBaseState(false)
     , m_selectionChanged(false)
     , m_backendValues(nullptr)
     , m_qmlComponent(nullptr)
     , m_qmlContext(nullptr)
+    , m_quickWidget(widget)
 {}
 
 QString PropertyEditorContextObject::convertColorToString(const QVariant &color)
 {
     QString colorString;
     QColor theColor;
-    if (color.canConvert(QVariant::Color)) {
+    if (color.canConvert(QMetaType(QMetaType::QColor))) {
         theColor = color.value<QColor>();
-    } else if (color.canConvert(QVariant::Vector3D)) {
+    } else if (color.canConvert(QMetaType(QMetaType::QVector3D))) {
         auto vec = color.value<QVector3D>();
         theColor = QColor::fromRgbF(vec.x(), vec.y(), vec.z());
     }
@@ -123,7 +70,7 @@ QString PropertyEditorContextObject::convertColorToString(const QVariant &color)
 
 QColor PropertyEditorContextObject::colorFromString(const QString &colorString)
 {
-    return convertColorFromString(colorString);
+    return QColor::fromString(colorString);
 }
 
 QString PropertyEditorContextObject::translateFunction()
@@ -148,10 +95,10 @@ QStringList PropertyEditorContextObject::autoComplete(const QString &text, int p
         return  Utils::filtered(m_model->rewriterView()->autoComplete(text, pos, explicitComplete), [filter](const QString &string) {
             return !filter || (!string.isEmpty() && string.at(0).isUpper()); });
 
-    return QStringList();
+    return {};
 }
 
-void PropertyEditorContextObject::toogleExportAlias()
+void PropertyEditorContextObject::toggleExportAlias()
 {
     QTC_ASSERT(m_model && m_model->rewriterView(), return);
 
@@ -169,13 +116,13 @@ void PropertyEditorContextObject::toogleExportAlias()
         PropertyName modelNodeId = selectedNode.id().toUtf8();
         ModelNode rootModelNode = rewriterView->rootModelNode();
 
-        rewriterView->executeInTransaction("PropertyEditorContextObject:toogleExportAlias", [&objectNode, &rootModelNode, modelNodeId](){
-            if (!objectNode.isAliasExported())
-                objectNode.ensureAliasExport();
-            else
-                if (rootModelNode.hasProperty(modelNodeId))
-                    rootModelNode.removeProperty(modelNodeId);
-        });
+        rewriterView->executeInTransaction("PropertyEditorContextObject:toggleExportAlias",
+                                           [&objectNode, &rootModelNode, modelNodeId]() {
+                                               if (!objectNode.isAliasExported())
+                                                   objectNode.ensureAliasExport();
+                                               else if (rootModelNode.hasProperty(modelNodeId))
+                                                   rootModelNode.removeProperty(modelNodeId);
+                                           });
     }
 }
 
@@ -202,27 +149,27 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
      * If we add more code here we have to forward the property editor view */
     RewriterView *rewriterView = m_model->rewriterView();
 
-    QTC_ASSERT(!rewriterView->selectedModelNodes().isEmpty(), return);
-
-    try {
-        auto transaction = RewriterTransaction(rewriterView, "PropertyEditorContextObject:changeTypeName");
-
-        ModelNode selectedNode = rewriterView->selectedModelNodes().constFirst();
+    QTC_ASSERT(!m_editorNodes.isEmpty(), return);
 
+    auto changeNodeTypeName = [&](ModelNode &selectedNode) {
         // Check if the requested type is the same as already set
         if (selectedNode.simplifiedTypeName() == typeName)
             return;
 
         NodeMetaInfo metaInfo = m_model->metaInfo(typeName.toLatin1());
         if (!metaInfo.isValid()) {
-            Core::AsynchronousMessageBox::warning(tr("Invalid Type"), tr("%1 is an invalid type.").arg(typeName));
+            Core::AsynchronousMessageBox::warning(tr("Invalid Type"),
+                                                  tr("%1 is an invalid type.").arg(typeName));
             return;
         }
 
         // Create a list of properties available for the new type
-        QList<PropertyName> propertiesAndSignals(metaInfo.propertyNames());
+        auto propertiesAndSignals = Utils::transform<PropertyNameList>(
+            PropertyEditorUtils::filteredProperties(metaInfo), &PropertyMetaInfo::name);
         // Add signals to the list
-        for (const auto &signal : metaInfo.signalNames()) {
+
+        const PropertyNameList &signalNames = metaInfo.signalNames();
+        for (const PropertyName &signal : signalNames) {
             if (signal.isEmpty())
                 continue;
 
@@ -234,14 +181,15 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
         }
 
         // Add dynamic properties and respective change signals
-        for (const auto &property : selectedNode.properties()) {
+        const QList<AbstractProperty> &nodeProperties = selectedNode.properties();
+        for (const AbstractProperty &property : nodeProperties) {
             if (!property.isDynamic())
                 continue;
 
             // Add dynamic property
-            propertiesAndSignals.append(property.name());
+            propertiesAndSignals.append(property.name().toByteArray());
             // Add its change signal
-            PropertyName name = property.name();
+            PropertyName name = property.name().toByteArray();
             QChar firstChar = QChar(property.name().at(0)).toUpper().toLatin1();
             name[0] = firstChar.toLatin1();
             name.prepend("on");
@@ -251,18 +199,21 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
 
         // Compare current properties and signals with the once available for change type
         QList<PropertyName> incompatibleProperties;
-        for (const auto &property : selectedNode.properties()) {
+        for (const AbstractProperty &property : nodeProperties) {
             if (!propertiesAndSignals.contains(property.name()))
-                incompatibleProperties.append(property.name());
+                incompatibleProperties.append(property.name().toByteArray());
         }
 
+        CompatibleProperties compatibleProps(selectedNode.metaInfo(), metaInfo);
+        compatibleProps.createCompatibilityMap(incompatibleProperties);
+
         Utils::sort(incompatibleProperties);
 
         // Create a dialog showing incompatible properties and signals
         if (!incompatibleProperties.empty()) {
             QString detailedText = QString("<b>Incompatible properties:</b><br>");
 
-            for (const auto &p : qAsConst(incompatibleProperties))
+            for (const auto &p : std::as_const(incompatibleProperties))
                 detailedText.append("- " + QString::fromUtf8(p) + "<br>");
 
             detailedText.chop(QString("<br>").size());
@@ -271,25 +222,46 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
             msgBox.setTextFormat(Qt::RichText);
             msgBox.setIcon(QMessageBox::Question);
             msgBox.setWindowTitle("Change Type");
-            msgBox.setText(QString("Changing the type from %1 to %2 can't be done without removing incompatible properties.<br><br>%3")
-                                   .arg(selectedNode.simplifiedTypeName())
-                                   .arg(typeName)
-                                   .arg(detailedText));
-            msgBox.setInformativeText("Do you want to continue by removing incompatible properties?");
+            msgBox.setText(QString("Changing the type from %1 to %2 can't be done without removing "
+                                   "incompatible properties.<br><br>%3")
+                               .arg(selectedNode.simplifiedTypeName(), typeName, detailedText));
+            msgBox.setInformativeText(
+                "Do you want to continue by removing incompatible properties?");
             msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
             msgBox.setDefaultButton(QMessageBox::Ok);
 
             if (msgBox.exec() == QMessageBox::Cancel)
                 return;
 
-            for (const auto &p : qAsConst(incompatibleProperties))
+            for (const auto &p : std::as_const(incompatibleProperties))
                 selectedNode.removeProperty(p);
         }
 
+        compatibleProps.copyMappedProperties(selectedNode);
+#ifdef QDS_USE_PROJECTSTORAGE
         if (selectedNode.isRootNode())
-            rewriterView->changeRootNodeType(metaInfo.typeName(), metaInfo.majorVersion(), metaInfo.minorVersion());
+            rewriterView->changeRootNodeType(typeName.toUtf8(), -1, -1);
         else
-            selectedNode.changeType(metaInfo.typeName(), metaInfo.majorVersion(), metaInfo.minorVersion());
+            selectedNode.changeType(typeName.toUtf8(), -1, -1);
+#else
+        if (selectedNode.isRootNode())
+            rewriterView->changeRootNodeType(metaInfo.typeName(),
+                                             metaInfo.majorVersion(),
+                                             metaInfo.minorVersion());
+        else
+            selectedNode.changeType(metaInfo.typeName(),
+                                    metaInfo.majorVersion(),
+                                    metaInfo.minorVersion());
+#endif
+        compatibleProps.applyCompatibleProperties(selectedNode);
+    };
+
+    try {
+        auto transaction = RewriterTransaction(rewriterView, "PropertyEditorContextObject:changeTypeName");
+
+        ModelNodes selectedNodes = m_editorNodes;
+        for (ModelNode &selectedNode : selectedNodes)
+            changeNodeTypeName(selectedNode);
 
         transaction.commit();
     } catch (const Exception &e) {
@@ -312,7 +284,7 @@ void PropertyEditorContextObject::insertKeyframe(const QString &propertyName)
 
     ModelNode selectedNode = rewriterView->selectedModelNodes().constFirst();
 
-    QmlTimeline timeline = rewriterView->currentTimeline();
+    QmlTimeline timeline = rewriterView->currentTimelineNode();
 
     QTC_ASSERT(timeline.isValid(), return );
     QTC_ASSERT(selectedNode.isValid(), return );
@@ -424,16 +396,127 @@ QQmlComponent *PropertyEditorContextObject::specificQmlComponent()
 
     m_qmlComponent = new QQmlComponent(m_qmlContext->engine(), this);
 
-    m_qmlComponent->setData(m_specificQmlData.toUtf8(), QUrl::fromLocalFile(QStringLiteral("specfics.qml")));
+    m_qmlComponent->setData(m_specificQmlData.toUtf8(), QUrl::fromLocalFile("specifics.qml"));
+
+    const bool showError = qEnvironmentVariableIsSet(Constants::ENVIRONMENT_SHOW_QML_ERRORS);
+    if (showError && !m_specificQmlData.isEmpty() && !m_qmlComponent->errors().isEmpty()) {
+        const QString errMsg = m_qmlComponent->errors().constFirst().toString();
+        Core::AsynchronousMessageBox::warning(tr("Invalid QML source"), errMsg);
+    }
 
     return m_qmlComponent;
 }
 
+bool PropertyEditorContextObject::hasMultiSelection() const
+{
+    return m_hasMultiSelection;
+}
+
+void PropertyEditorContextObject::setHasMultiSelection(bool b)
+{
+    if (b == m_hasMultiSelection)
+        return;
+
+    m_hasMultiSelection = b;
+    emit hasMultiSelectionChanged();
+}
+
+bool PropertyEditorContextObject::isSelectionLocked() const
+{
+    return m_isSelectionLocked;
+}
+
+void PropertyEditorContextObject::setIsSelectionLocked(bool lock)
+{
+    if (lock == m_isSelectionLocked)
+        return;
+
+    m_isSelectionLocked = lock;
+    emit isSelectionLockedChanged();
+}
+
+void PropertyEditorContextObject::setInsightEnabled(bool value)
+{
+    if (value != m_insightEnabled) {
+        m_insightEnabled = value;
+        emit insightEnabledChanged();
+    }
+}
+
+void PropertyEditorContextObject::setInsightCategories(const QStringList &categories)
+{
+    m_insightCategories = categories;
+    emit insightCategoriesChanged();
+}
+
+bool PropertyEditorContextObject::hasQuick3DImport() const
+{
+    return m_hasQuick3DImport;
+}
+
+void PropertyEditorContextObject::setEditorNodes(const ModelNodes &nodes)
+{
+    m_editorNodes = nodes;
+}
+
+void PropertyEditorContextObject::setHasQuick3DImport(bool value)
+{
+    if (value == m_hasQuick3DImport)
+        return;
+
+    m_hasQuick3DImport = value;
+    emit hasQuick3DImportChanged();
+}
+
+bool PropertyEditorContextObject::hasMaterialLibrary() const
+{
+    return m_hasMaterialLibrary;
+}
+
+void PropertyEditorContextObject::setHasMaterialLibrary(bool value)
+{
+    if (value == m_hasMaterialLibrary)
+        return;
+
+    m_hasMaterialLibrary = value;
+    emit hasMaterialLibraryChanged();
+}
+
+bool PropertyEditorContextObject::isQt6Project() const
+{
+    return m_isQt6Project;
+}
+
+void PropertyEditorContextObject::setIsQt6Project(bool value)
+{
+    if (m_isQt6Project == value)
+        return;
+
+    m_isQt6Project = value;
+    emit isQt6ProjectChanged();
+}
+
+bool PropertyEditorContextObject::has3DModelSelected() const
+{
+    return m_has3DModelSelected;
+}
+
+void PropertyEditorContextObject::setHas3DModelSelected(bool value)
+{
+    if (value == m_has3DModelSelected)
+        return;
+
+    m_has3DModelSelected = value;
+    emit has3DModelSelectedChanged();
+}
+
 void PropertyEditorContextObject::setSpecificsUrl(const QUrl &newSpecificsUrl)
 {
     if (newSpecificsUrl == m_specificsUrl)
         return;
 
+    qCInfo(urlSpecifics) << Q_FUNC_INFO << newSpecificsUrl;
+
     m_specificsUrl = newSpecificsUrl;
     emit specificsUrlChanged();
 }
@@ -557,8 +640,7 @@ int PropertyEditorContextObject::devicePixelRatio()
 
 QStringList PropertyEditorContextObject::styleNamesForFamily(const QString &family)
 {
-    const QFontDatabase dataBase;
-    return dataBase.styles(family);
+    return QFontDatabase::styles(family);
 }
 
 QStringList PropertyEditorContextObject::allStatesForId(const QString &id)
@@ -576,16 +658,51 @@ bool PropertyEditorContextObject::isBlocked(const QString &propName) const
 {
     if (m_model && m_model->rewriterView()) {
         const QList<ModelNode> nodes = m_model->rewriterView()->selectedModelNodes();
-        QScopedPointer<QmlObjectNode> objNode;
         for (const auto &node : nodes) {
-            objNode.reset(QmlObjectNode::getQmlObjectNodeOfCorrectType(node));
-            if (objNode->isBlocked(propName.toUtf8()))
+              if (Qml3DNode qml3DNode{node}; qml3DNode.isBlocked(propName.toUtf8()))
                 return true;
         }
     }
     return false;
 }
 
+void PropertyEditorContextObject::verifyInsightImport()
+{
+    Import import = Import::createLibraryImport("QtInsightTracker", "1.0");
+
+    if (!m_model->hasImport(import))
+        m_model->changeImports({import}, {});
+}
+
+QRect PropertyEditorContextObject::screenRect() const
+{
+    if (m_quickWidget && m_quickWidget->screen())
+        return m_quickWidget->screen()->availableGeometry();
+    return  {};
+}
+
+QPoint PropertyEditorContextObject::globalPos(const QPoint &point) const
+{
+    if (m_quickWidget)
+        return m_quickWidget->mapToGlobal(point);
+    return point;
+}
+
+void PropertyEditorContextObject::handleToolBarAction(int action)
+{
+    emit toolBarAction(action);
+}
+
+void PropertyEditorContextObject::saveExpandedState(const QString &sectionName, bool expanded)
+{
+    s_expandedStateHash.insert(sectionName, expanded);
+}
+
+bool PropertyEditorContextObject::loadExpandedState(const QString &sectionName, bool defaultValue) const
+{
+    return s_expandedStateHash.value(sectionName, defaultValue);
+}
+
 void EasingCurveEditor::registerDeclarativeType()
 {
      qmlRegisterType<EasingCurveEditor>("HelperWidgets", 2, 0, "EasingCurveEditor");
